@@ -1,3 +1,17 @@
+dataCalcVirtual <- nimbleFunctionVirtual(
+  run = function() 
+    returnType(double())
+)
+
+calcDataNF <- nimbleFunction(
+  contains = dataCalcVirtual,
+  setup = function(model, node){},
+  run = function(){
+    returnType(double())
+    return(model$calculate(node))
+  }, where = getLoadingNamespace()
+)
+
 
 #' Create an MCMC function, from an MCMCconf object
 #'
@@ -32,7 +46,16 @@
 #' \code{Cmcmc <- compileNimble(Rmcmc, project=Rmodel)}
 #'
 #' The compiled function will function identically to the uncompiled object, except acting on the compiled model object.
+#'
+#' @section Calculating WAIC:
+#' After the MCMC has been run, calling the \code{calculateWAIC()} method of the MCMC object will return the WAIC for the model, calculated using the posterior samples from the MCMC run.
 #' 
+#' \code{calculateWAIC()} has a single arugment:
+#'
+#' \code{burnIn}: The number of iterations to subtract from the beginning of the posterior samples of the MCMC object for WAIC calculation.  Defaults to 0.
+#' 
+#' The \code{calculateWAIC} method calculates the WAIC of the model that the MCMC was performed on. The WAIC (Watanabe, 2010) is calculated from Equations 5, 12, and 13 in Gelman (2014).  Note that the set of all parameters monitored by the mcmc object will be treated as \eqn{theta} for the purposes of e.g. Equation 5 from Gelman (2014).  All parameters downstream of the monitored parameters that are necessary to calculate \eqn{p(y|theta)} will be simulated from the posterior samples of \eqn{theta}.
+#'
 #' @examples
 #' \dontrun{
 #' code <- nimbleCode({
@@ -47,8 +70,10 @@
 #' Cmcmc$run(10000)
 #' samples <- as.matrix(Cmcmc$mvSamples)
 #' head(samples)
+#' WAIC <- Cmcmc$calculateWAIC(burnIn = 1000)
 #' }
 buildMCMC <- nimbleFunction(
+    name = 'MCMC',
     setup = function(conf, ...) {
     	if(inherits(conf, 'modelBaseClass'))
     		conf <- configureMCMC(conf, ...)
@@ -73,7 +98,17 @@ buildMCMC <- nimbleFunction(
         samplerTimes <- c(0,0) ## establish as a vector
         progressBarLength <- 52  ## multiples of 4 only
         progressBarDefaultSetting <- getNimbleOption('MCMCprogressBar')
-    },
+        
+        ## WAIC setup below:
+        dataNodes <- model$getNodeNames(dataOnly = TRUE)
+        dataCalcNFList <- nimbleFunctionList(dataCalcVirtual)
+        for(i in seq_along(dataNodes)){
+          dataCalcNFList[[i]] <- calcDataNF(model, dataNodes[i])
+        }
+        dataNodeLength <- length(dataNodes)
+        sampledNodes <- model$getVarNames(FALSE, colnames(as.matrix(mvSamples)))
+        paramDeps <- model$getDependencies(sampledNodes, self = FALSE)
+        },
 
     run = function(niter = integer(), reset = logical(default=TRUE), simulateAll = logical(default=FALSE), time = logical(default=FALSE), progressBar = logical(default=TRUE)) {
         if(simulateAll)     simulate(model)    ## default behavior excludes data nodes
@@ -122,6 +157,34 @@ buildMCMC <- nimbleFunction(
         getTimes = function() {
             returnType(double(1))
             return(samplerTimes[1:(length(samplerTimes)-1)])
+        },
+        calculateWAIC = function(burnIn = integer(default = 0)){
+            numMCMCSamples <- getsize(mvSamples) - burnIn
+            if((numMCMCSamples) < 2){
+              print('Error, need more than 1 post burn-in MCMC sample.')
+              return(-Inf)
+            }
+            logPredProbs <- numeric(numMCMCSamples)
+            logAvgProb <- 0
+            pWAIC <- 0
+            currentVals <- values(model, sampledNodes)
+            for(i in 1:dataNodeLength){
+              meanPredProb <- 0
+              for(j in 1:numMCMCSamples){
+                copy(mvSamples, model, nodesTo = sampledNodes, row = j + burnIn)
+                model$simulate(paramDeps)
+                logPredProbs[j] <- dataCalcNFList[[i]]$run()
+              }
+              meanPredProb <- mean(exp(logPredProbs))
+              pointLogPredVar <- sd(logPredProbs)^2
+              pWAIC <- pWAIC + pointLogPredVar
+              logAvgProb <- logAvgProb + log(meanPredProb)
+            }
+            WAIC <- -2*(logAvgProb- pWAIC)
+            values(model, sampledNodes) <<- currentVals
+            model$calculate(paramDeps)
+            returnType(double())
+            return(WAIC)
         }),
     where = getLoadingNamespace()
 )
