@@ -12,6 +12,8 @@ otherDSLcalls <- c("{",
                    "nimEigen",
                    "nimSvd",
                    "nimOptim",
+                   "nimOptimDefaultControl",
+                   "nimDerivs",
                    "void")
 
 nimKeyWords <- list(copy = 'nimCopy',
@@ -32,7 +34,9 @@ nimKeyWords <- list(copy = 'nimCopy',
                     seq = 'nimSeq',
                     eigen = 'nimEigen',
                     svd = 'nimSvd',
-                    optim = 'nimOptim')
+                    optim = 'nimOptim',
+                    optimDefaultControl = 'nimOptimDefaultControl',
+                    derivs = 'nimDerivs')
 
 nfMethodRCinterface <- setRefClass(Class = 'nfMethodRCinterface',
                                    fields = list(
@@ -46,7 +50,9 @@ nfMethodRC <-
                 fields  = list(
                     template   = 'ANY',
                     code       = 'ANY',
-                    neededRCfuns = 'ANY'		#list
+                    neededRCfuns = 'ANY',		#list
+                    externalHincludes = 'ANY',
+                    externalCPPincludes = 'ANY'
                 ),
                 methods = list(
                     initialize = function(method, name, check = FALSE, methodNames = NULL, setupVarNames = NULL) {
@@ -60,6 +66,8 @@ nfMethodRC <-
                         generateArgs()
                         generateTemplate() ## used for argument matching
                         removeAndSetReturnType()
+                        ## New system for external calls:
+                        externalHincludes <<- externalCPPincludes <<- list() ## to be populated by nimbleExternalCall
                     },
                     generateArgs = function() {
                         argsList <- nf_createAList(names(argInfo))
