@@ -1,7 +1,4 @@
 ## Classes for a cppProject
-setOldClass("DLLInfo")
-setClassUnion("DLLInfoOrNULL", c("NULL", "DLLInfo"))
-
 cppCodeFileClass <- setRefClass('cppCodeFileClass',
                              fields = list(
                                  filename = 'ANY',	#character
@@ -105,7 +102,6 @@ cppProjectClass <- setRefClass('cppProjectClass',
                                    addFunction = function(funDef, name, filename) {
                                        if(missing(name)) name <- funDef$name
                                        cppDefs[[name]] <<- funDef
-                                         ##XXX This computation doesn't seem to matter. Where is filename stored? ANS: There is a field in the funDef ref class object for it.  could be done in 1 line instead of 2
                                        if(!missing(filename)) {
                                            filename <- Rname2CppName(filename); funDef$filename <- filename
                                        } else {
@@ -144,15 +140,19 @@ cppProjectClass <- setRefClass('cppProjectClass',
                                        selfCPP <- if(is.character(con)) paste0('"', con, '.cpp"') else '"[FILENAME].cpp"'
                                        CPPincludes <- CPPincludes[ CPPincludes != selfCPP ]
 
-                                       ## Eigen must be included before any R header files because they both define "length"
+                                      ## TODO Simplify Eigen include logic now that Nimble defines `R_NO_REMAP`.
+                                       ## similar for cppad
                                        iEigenInclude <- grep("EigenTypedefs", CPPincludes)
                                        if(length(iEigenInclude) > 0) {
                                            CPPincludes <- c(CPPincludes[iEigenInclude], CPPincludes[-iEigenInclude])
                                        }
+                                       iCppInclude <- grep("cppad", CPPincludes)
+                                       if(length(iCppInclude) > 0) {
+                                           CPPincludes <- c(CPPincludes[iCppInclude], CPPincludes[-iCppInclude])
+                                       }
 
                                        ## at this point strip out CPPincludes other than EigenTypedefs that have .cpp and gsub .cpp to .o
                                        boolConvertCppIncludeToOinclude <- grepl("\\.cpp", CPPincludes)
-                                       if(length(iEigenInclude) > 0) boolConvertCppIncludeToOinclude[1] <- FALSE
                                        Oincludes <<- gsub("\\.cpp", ".o", CPPincludes[boolConvertCppIncludeToOinclude])
                                        CPPincludes <- CPPincludes[!boolConvertCppIncludeToOinclude]
 
@@ -167,9 +167,7 @@ cppProjectClass <- setRefClass('cppProjectClass',
                                                               cppDefs = cppPieces,
                                                               ifndefName = ifndefName)
                                        selfInclude <- if(is.character(con)) paste0('"', con, '.h', '"') else '"[FILENAME].h"'
-                                       CPPincludes <- c(CPPincludes, selfInclude) ## selfInclude has to come last because Rinternals.h makes a name conflict with Eigen
-                                       
-                                     ##  CPPincludes <- c(CPPincludes, ('\"nimble/dynamicRegistrations.cpp\"'))
+                                       CPPincludes <- c(CPPincludes, selfInclude) ## selfInclude has to come last because Rinternals.h makes a name conflict with Eigen (this may be moot, 7/17)
                                        
                                        cppIfndefName <- paste0(ifndefName,'_CPP')
                                        cppFile <- cppCPPfileClass(filename = filename,
@@ -202,78 +200,83 @@ cppProjectClass <- setRefClass('cppProjectClass',
                                            "}")
                                        writeLines(contentLines, con = dynamicRegistrationsCppName)
                                    },
-                                   compileFile = function(names, showCompilerOutput = nimbleOptions('showCompilerOutput'),
-                                                          .useLib = UseLibraryMakevars) {
-                                       cppPermList <- c('RcppUtils.cpp',
-                                                        'Utils.cpp',
-                                                        'NamedObjects.cpp',
-                                                        'ModelClassUtils.cpp',
-                                                        'accessorClasses.cpp',
-                                                        'predefinedNimbleLists.cpp',
-                                                        'nimOptim.cpp'
-                                                        )
-                                       if(getNimbleOption('includeCPPdists')) cppPermList <- c(cppPermList, 'dists.cpp', 'nimDists.cpp')
-
-                                       isWindows = (.Platform$OS.type == "windows")
-
-                                       includes <- character()
-
-                                       ## following was before we created libnimble.a as an alternative to libnimble.so/libnimble.dll
-                                       ## includes <- if(!.useLib) {
-	                               ##                if(isWindows) {
-                                       ##                   shortDirname = dirname(shortPathName(sprintf("%s/%s", NimbleCodeDir, cppPermList[1])))
-		    		       ##                   sprintf("%s/%s", shortDirname, cppPermList)
-                                       ##                } else
-                                       ##                   sprintf("%s/%s", normalizePath(NimbleCodeDir, winslash = '/'), cppPermList)
-                                       ## 	            } else
-                                       ##                 character()
-
+                                   compileDynamicRegistrations = function(showCompilerOutput = nimbleOptions('showCompilerOutput')) {
                                        timeStamp <- format(Sys.time(), "%m_%d_%H_%M_%S")
-
+                                       
                                        dynamicRegistrationsDllName <- paste0("dynamicRegistrations_", timeStamp)
                                        dynamicRegistrationsCppName <- paste0(dynamicRegistrationsDllName, ".cpp")
                                        
-                                       ## mainfiles <- paste(paste(basename(file.path(dirName, paste0(names,'.cpp'))), collapse = ' '), dynamicRegistrationsCppName)
-                                       mainfiles <- paste(basename(file.path(dirName, paste0(names,'.cpp'))), collapse = ' ')
+                                       writeDynamicRegistrationsDotCpp(dynamicRegistrationsCppName, dynamicRegistrationsDllName)
+                                       ssDllName <- file.path(dirName, paste0(dynamicRegistrationsDllName, .Platform$dynlib.ext))
+                                       ssdSHLIBcmd <- paste(file.path(R.home('bin'), 'R'), 'CMD SHLIB', dynamicRegistrationsCppName, '-o', basename(ssDllName))
+                                       if(!showCompilerOutput) {
+                                           logFile <- paste0("dynamicRegistrations_", format(Sys.time(), "%m_%d_%H_%M_%S"), ".log")
+                                           ssdSHLIBcmd <- paste(ssdSHLIBcmd, ">", logFile)
+                                           ## Rstudio will fail to run a system() command with show.output.on.console=FALSE if any output is actually directed to the console. Redirecting it to a file seems to cure this.
+                                       }
+                                       isWindows = (.Platform$OS.type == "windows")
+                                       if(isWindows)
+                                           status = system(ssdSHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput, show.output.on.console = showCompilerOutput)
+                                       else
+                                           status = system(ssdSHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput)
+                                       if(status != 0) 
+                                           stop(structure(simpleError("Failed to create the shared library"),
+                                                          class = c("SHLIBCreationError", "ShellError", "simpleError", "error", "condition")))
+                                       nimbleUserNamespace$sessionSpecificDll <- dyn.load(ssDllName, local = TRUE)
+                                   },                                  
+                                   compileFile = function(names, showCompilerOutput = nimbleOptions('showCompilerOutput'),
+                                                          .useLib = UseLibraryMakevars) {
+                                       isWindows = (.Platform$OS.type == "windows")
 
+                                       includes <- character()
+                                       timeStamp <- format(Sys.time(), "%m_%d_%H_%M_%S")
+
+                                       mainfiles <- paste(basename(file.path(dirName, paste0(names,'.cpp'))), collapse = ' ')
 
 				       if(!file.exists(file.path(dirName, sprintf("Makevars%s", if(isWindows) ".win" else ""))) && NeedMakevarsFile) # should reverse the order here in the long term.
 				           createMakevars(.useLib = .useLib, dir = dirName)
 
                                        dllName <- paste0(names[1], "_", timeStamp)
-                                       
+                                                                             
                                        outputSOfile <<- file.path(dirName, paste0(dllName, .Platform$dynlib.ext))
 
-                                       includes <- c(includes, Oincludes)
+                                       if(!inherits(Oincludes, 'uninitializedField')) { ## will only be uninitialized if writeFiles was skipped due to specialHandling (developer backdoor)
+                                           includes <- c(includes, Oincludes) ## normal operation will have Oincludes.
+                                       }
                                        SHLIBcmd <- paste(file.path(R.home('bin'), 'R'), 'CMD SHLIB', paste(c(mainfiles, includes), collapse = ' '), '-o', basename(outputSOfile))
 
                                        cur = getwd()
                                        setwd(dirName)
                                        on.exit(setwd(cur))
 
-                                       if(is.null(nimbleUserNamespace$sessionSpecificDll)) {
-                                           writeDynamicRegistrationsDotCpp(dynamicRegistrationsCppName, dynamicRegistrationsDllName)
-                                           ssDllName <- file.path(dirName, paste0(dynamicRegistrationsDllName, .Platform$dynlib.ext))
-                                           ssdSHLIBcmd <- paste(file.path(R.home('bin'), 'R'), 'CMD SHLIB', dynamicRegistrationsCppName, '-o', basename(ssDllName))
-                                           if(!showCompilerOutput) {
-                                               logFile <- paste0("dynamicRegistrations_", format(Sys.time(), "%m_%d_%H_%M_%S"), ".log")
-                                               ssdSHLIBcmd <- paste(ssdSHLIBcmd, ">", logFile)
-                                               ## Rstudio will fail to run a system() command with show.output.on.console=FALSE if any output is actually directed to the console. Redirecting it to a file seems to cure this.
-                                           }
-                                           if(isWindows)
-                                               status = system(ssdSHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput, show.output.on.console = showCompilerOutput)
-                                           else
-                                               status = system(ssdSHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput)
-                                           if(status != 0) 
-                                               stop(structure(simpleError("Failed to create the shared library"),
-                                                              class = c("SHLIBCreationError", "ShellError", "simpleError", "error", "condition")))
-                                           nimbleUserNamespace$sessionSpecificDll <- dyn.load(ssDllName, local = TRUE)
+                                       if(is.null(nimbleUserNamespace$sessionSpecificDll)) compileDynamicRegistrations(showCompilerOutput = showCompilerOutput)
+
+                                       origSHLIBcmd <- SHLIBcmd
+                                       if(isTRUE(nimbleOptions('stopCompilationBeforeLinking'))) {## used only for testing, when we want to go quickly and skip linking and bail out
+                                           ## get the dry run commands, run only those that contain -c for compile-only (don't link)
+                                           ## this has only been tested with single .cpp files, not multiple .cpp files
+                                           dryRunCmd <- paste0(SHLIBcmd, " -n")
+                                           dryRunResult <- system(dryRunCmd, intern = TRUE)
+                                           compileOnlyLines <- dryRunResult[ grepl("-c", dryRunResult) ]
+                                           SHLIBcmd <- paste0(compileOnlyLines, collapse =  ";" )
                                        }
 
+                                       if(isTRUE(nimbleOptions('forceO1'))) { ## replace -On flags with -O1 to reduce compiler time due to higher optimization levels 
+                                           ## If force01 is TRUE and we did not already strip out -c flags, do so now
+                                           if(!isTRUE(nimbleOptions('stopCompilationBeforeLinking'))) {
+                                               dryRunCmd <- paste0(SHLIBcmd, " -n")
+                                               dryRunResult <- system(dryRunCmd, intern = TRUE)
+                                               compileOnlyLines <- dryRunResult[ grepl("-c", dryRunResult) ]
+                                               SHLIBcmd <- paste0(compileOnlyLines, collapse =  ";" )
+                                           }
+                                           SHLIBcmd <- gsub("-O[1-9]", "-O1", SHLIBcmd)
+                                           SHLIBcmd <- paste0(SHLIBcmd, "; ", origSHLIBcmd)
+                                       }
+                                       
                                        if(!showCompilerOutput) { 
                                            logFile <- paste0(names[1], "_", format(Sys.time(), "%m_%d_%H_%M_%S"), ".log")
                                            SHLIBcmd <- paste(SHLIBcmd, ">", logFile)
-                                           ## Rstudio will fail to run a system() command with show.output.on.console=FALSE if any output is actually directed to the console. Redirecting it to a file seems to cure this.
+                                           ## See Rstudio comment above
                                        }
 
                                        if(nimbleOptions('pauseAfterWritingFiles')) browser()
@@ -283,8 +286,10 @@ cppProjectClass <- setRefClass('cppProjectClass',
                                        else
                                            status = system(SHLIBcmd, ignore.stdout = !showCompilerOutput, ignore.stderr = !showCompilerOutput)
 				       if(status != 0)
-                                          stop(structure(simpleError("Failed to create the shared library"),
-                                                         class = c("SHLIBCreationError", "ShellError", "simpleError", "error", "condition")))
+                                           stop(structure(simpleError("Failed to create the shared library"),
+                                                          class = c("SHLIBCreationError", "ShellError", "simpleError", "error", "condition")))
+                                       if(isTRUE(nimbleOptions()$stopCompilationBeforeLinking)) stop("safely stopping before linking", call.=FALSE)
+
                                    },
                                    loadSO = function(name) {
                                        dll <<- dyn.load(getSOName(name, dirName), local = TRUE)
