@@ -18,7 +18,6 @@ cppVirtualNimbleFunctionClass <- setRefClass('cppVirtualNimbleFunctionClass',
                                                          else {
                                                              baseClassName <- environment(baseClassObj)$CclassName
                                                              addInheritance(baseClassName)
-                                                            ## addAncestors(c(baseClassObj$inheritance, baseClassObj$ancestors))
                                                          }
                                                      }
                                                  },
@@ -39,7 +38,7 @@ cppNimbleClassClass <- setRefClass('cppNimbleClassClass',
                                        ## Inherits a functionDefs list for member functions
                                        ## Inherits an objectDefs list for member data
                                        SEXPmemberInterfaceFuns = 'ANY', ## List of SEXP interface functions, one for each member function
-                                       nimCompProc = 'ANY', ## an nfProcessing or nlProcessing class, needed to get the member data symbol table post-compilation
+                                       nimCompProc = 'ANY', ## nfProcessing or nlProcessing object, needed to get the member data symbol table post-compilation
 
                                        Rgenerator = 'ANY' , ## function to generate and wrap a new object from an R object
                                        CmultiInterface = 'ANY', ## object for interfacing multiple C instances when a top-level interface is not needed
@@ -189,7 +188,8 @@ cppNimbleClassClass <- setRefClass('cppNimbleClassClass',
 cppNimbleFunctionClass <- setRefClass('cppNimbleFunctionClass',
                                       contains = 'cppNimbleClassClass',
                                       fields = list(
-                                          nfProc = 'ANY' ## an nfProcessing class, needed to get the member data symbol table post-compilation
+                                          nfProc = 'ANY', ## an nfProcessing class, needed to get the member data symbol table post-compilation
+                                          parentsSizeAndDims = 'ANY'
                                           ),
                                           methods = list(
                                               getDefs = function() {
@@ -208,12 +208,12 @@ cppNimbleFunctionClass <- setRefClass('cppNimbleFunctionClass',
                                               initialize = function(nfProc, isNode, debugCpp = FALSE, fromModel = FALSE, ...) {
                                                   callSuper(nfProc, debugCpp, fromModel, ...)
                                                   if(!missing(nfProc)) processNFproc(nfProc, debugCpp = debugCpp, fromModel = fromModel)
-                                                  
                                                   if(isNode) {
                                                       inheritance <<- inheritance[inheritance != 'NamedObjects']
                                                       baseClassObj <- environment(nfProc$nfGenerator)$contains
                                                       if(is.null(baseClassObj)) {
                                                           inheritance <<- c(inheritance, 'nodeFun')
+                                                          parentsSizeAndDims <<- environment(nfProc$nfGenerator)$parentsSizeAndDims
                                                       }
                                                   }
                                               },
@@ -240,6 +240,77 @@ cppNimbleFunctionClass <- setRefClass('cppNimbleFunctionClass',
                                                       RCfunDefs[[RCname]] <<- functionDefs[[RCname]]
                                                   }
                                               },
+                                              addTypeTemplateFunction = function( funName ) {
+                                                  newFunName <- paste0(funName, '_AD_')
+                                                  regularFun <- RCfunDefs[[funName]]
+                                                  functionDefs[[newFunName]] <<- makeTypeTemplateFunction(newFunName, regularFun)
+                                                  invisible(NULL)
+                                              },
+                                              addADtapingFunction = function( funName, independentVarNames, dependentVarNames ) {
+                                                  ADfunName <- paste0(funName, '_AD_')
+                                                  regularFun <- RCfunDefs[[funName]]
+                                                  newFunName <- paste0(funName, '_callForADtaping_')
+                                                  functionDefs[[newFunName]] <<- makeADtapingFunction(newFunName, regularFun, ADfunName, independentVarNames, dependentVarNames, nfProc$isNode, functionDefs)
+                                                  invisible(NULL)
+                                              },
+                                              addADargumentTransferFunction = function( funName, independentVarNames ) {
+                                                  newFunName <- paste0(funName, '_ADargumentTransfer_')
+                                                  regularFun <- RCfunDefs[[funName]]
+                                                  funIndex <- which(environment(nfProc$nfGenerator)$enableDerivs == funName) ## needed for correct index for allADtapePtrs_
+                                                  functionDefs[[newFunName]] <<- makeADargumentTransferFunction(newFunName, regularFun, independentVarNames, funIndex, parentsSizeAndDims)
+                                              },
+                                              addStaticInitClass = function() {
+                                                  neededTypeDefs[['staticInitClass']] <<- makeStaticInitClass(.self, environment(nfProc$nfGenerator)$enableDerivs) ##
+                                                  invisible(NULL)
+                                              },
+                                              addADclassContentOneFun = function(funName) {
+                                                  outSym <- nfProc$RCfunProcs[[funName]]$compileInfo$returnSymbol
+                                                  checkADargument(funName, outSym, returnType = TRUE)
+                                                  if(length(nfProc$RCfunProcs[[funName]]$nameSubList) == 0) stop(paste0('Derivatives cannot be enabled for method ', funName, ', since this method has no arguments.'))
+                                                  if(!nfProc$isNode){
+                                                    for(iArg in seq_along(functionDefs[[funName]]$args$symbols)){
+                                                      arg <- functionDefs[[funName]]$args$symbols[[iArg]]
+                                                      argSym <- nfProc$RCfunProcs[[funName]]$compileInfo$origLocalSymTab$getSymbolObject(arg$name)
+                                                      argName <- names(nfProc$RCfunProcs[[funName]]$nameSubList)[iArg]
+                                                      checkADargument(funName, argSym, argName = argName)
+                                                    }
+                                                  }
+                                                  addTypeTemplateFunction(funName)
+                                                  independentVarNames <- names(functionDefs[[funName]]$args$symbols)
+                                                  if(nfProc$isNode) independentVarNames <- independentVarNames[-1]  ## remove ARG1_INDEXEDNODEINFO__ from independentVars
+                                                  
+                                                  addADtapingFunction(funName, independentVarNames = independentVarNames, dependentVarNames = 'ANS_' )
+                                                  addADargumentTransferFunction(funName, independentVarNames = independentVarNames)
+                                              },
+                                              checkADargument = function(funName, argSym, argName = NULL, returnType = FALSE){
+                                                argTypeText <- if(returnType) 'returnType' else 'argument'
+                                                if(argSym$type != 'double')
+                                                  stop(paste0('The ', argName, ' ', argTypeText, ' of the ', funName, ' method is not a double, this method cannot have derivatives enabled.'))
+                                                if(!(argSym$nDim %in% c(0,1)))
+                                                   stop(paste0('The ', argName, ' ', argTypeText, ' of the ', funName, ' method must be a double scalar or double vector for derivatives to be enabled.'))
+                                                if((argSym$nDim == 1) && is.na(argSym$size)) stop(paste0('To enable derivatives, size must be given for the ', argName, ' ', argTypeText, ' of the ', funName,
+                                                                                                          ' method,  e.g. double(1, 3) for a length 3 vector.' ))
+                                              },
+
+                                              addADclassContent = function() {
+                                                  CPPincludes <<- c("<cppad/cppad.hpp>", CPPincludes)
+                                                  Hincludes <<- c("<cppad/cppad.hpp>", nimbleIncludeFile("nimbleCppAD.h"), Hincludes)
+                                                  addInheritance("nimbleFunctionCppADbase")
+                                                  objectDefs$addSymbol(cppVarFull(baseType = 'vector', templateArgs = list(cppVarFull(baseType = 'CppAD::ADFun', templateArgs = list('double'), ptr = 1)), static = TRUE, name = 'allADtapePtrs_'))
+                                                  objectDefs$addSymbol(cppVarFull(name = 'ADtapeSetup', baseType = 'nimbleCppADinfoClass'))
+                                                  for(adEnabledFun in environment(nfProc$nfGenerator)$enableDerivs){
+                                                    addADclassContentOneFun(adEnabledFun)
+                                                  }
+                                                  ## static declaration in the class definition
+                                                  ## globals to hold the global static definition
+                                                  globals <- cppGlobalObjects(name = paste0('staticGlobals_', name), staticMembers = TRUE)
+                                                  globals$objectDefs[['staticGlobalTape']] <- cppVarFull(baseType = 'vector', templateArgs = list(cppVarFull(baseType = 'CppAD::ADFun', templateArgs = list('double'), ptr = 1)), name = paste0(name,'::allADtapePtrs_'))
+                                                  neededTypeDefs[['allADtapePtrs_']] <<- globals
+
+                                                  addStaticInitClass()
+
+                                                  invisible(NULL)
+                                              },
                                               buildCmultiInterface = function(dll = NULL) {
                                                   sym <- if(!is.null(dll))
                                                              getNativeSymbolInfo(SEXPgeneratorFun$name, dll)
@@ -265,7 +336,6 @@ cppNimbleFunctionClass <- setRefClass('cppNimbleFunctionClass',
                                                   newCobjectInterface
                                               },
                                               buildCallable = function(R_NimbleFxn, dll = NULL, asTopLevel = TRUE){
-                                             ##     cat('buildCallable\n')
                                                   if(asTopLevel) {
                                                       cppInterfaceObject <- Rgenerator(R_NimbleFxn, dll, project = nimbleProject)
                                                   } else { ## actually this particular pathway should never be taken. asTopLevel = FALSE will occur only for nimbleProject$instantiateNimbleFunction 
@@ -275,134 +345,16 @@ cppNimbleFunctionClass <- setRefClass('cppNimbleFunctionClass',
                                               },
                                               buildAll = function(where = where) {
                                                   baseClassObj <- environment(nfProc$nfGenerator)$contains
+                                                  
                                                   if(!is.null(baseClassObj)) {
                                                       inheritance <<- inheritance[inheritance != 'NamedObjects']
                                                       baseClassName <- environment(baseClassObj)$name
                                                       addInheritance(baseClassName)
-                                                      ## These lines would be completely general...
-                                                      ##baseClassCppDef <- environment(baseClassObj)$cppDef
-                                                      ##if(is.null(baseClassCppDef)) warning('cppDef for a base class not available when needed')
-                                                      ##addAncestors(c(baseClassCppDef$inheritance, baseClassCppDef$ancestors))
-                                                      ## ... but I'm going with this for now because it is just what is needed 
                                                       addAncestors('NamedObjects')
                                                   }
+                                                  if(nimbleOptions('experimentalEnableDerivs') && length(environment(nfProc$nfGenerator)$enableDerivs) > 0) addADclassContent()
+
                                                   callSuper(where)
                                               }
                                           ),
                                       )
-
-## This and the functions below were original stand-along compilation functions
-## Now they are superceded by compileNimble.
-## A stand-alone call could still work but these may have faller out of date.
-compileNimbleFunction <- function(fun, dirName, all.instances = TRUE, individual.instances = NA, name = deparse(substitute(fun) ),  fileName = Rname2CppName(name), writeFiles = !(environment(fun)$Cwritten), compileCpp = !(environment(fun)$compiled), loadSO = !(environment(fun)$loadedSO), debug = FALSE, debugCpp = FALSE, returnInternals = FALSE ) {
-    if(missing(dirName))    dirName <- makeDefaultDirName()
-    
-        if(!is.nfGenerator(fun)) stop('Error in compileNimbleFunction: fun should be a nimbleFunction generator')
-	cppProj <- nfWriteCompileAndLoadSO(RFun = fun, dirName = dirName, name = name, fileName = fileName,
-                                           writeFiles = writeFiles, compileCpp = compileCpp, loadSO = loadSO, debug = debug, debugCpp = debugCpp)
-
-        if(returnInternals) return(cppProj)
-        
-	if(all.instances == FALSE){
-            if(!is.list(individual.instances))
-                return(nfBuildCInterface(cppProj, instance = individual.instances, name = name) ) 
-            else{
-                functionList = list()
-                for(i in 1:length(individual.instances) ) 
-                    functionList[[i]] <- nfBuildCInterface(cppProj, instance = individual.instances[[i]], name = name)
-            }
-            return(functionList)
-	}
-
-	functionList = list()
-	k = length(environment(fun)$instances)
-	if(k == 1)
-            {
-		nfRefClassObject <- environment(fun)$instances[[1]]
-		wrappedFunction <- function(...) { nfRefClassObject$run() }
-		environment(wrappedFunction) <- new.env(parent = parent.frame())
-		environment(wrappedFunction)$nfRefClassObject <- nfRefClassObject
-		return(nfBuildCInterface(cppProj, instance = wrappedFunction, name = name) )
-            }
-	
-	for(i in 1:k) {
-            nfRefClassObject <- environment(fun)$instances[[i]]
-            wrappedFunction <- function(...) { nfRefClassObject$run() }
-            environment(wrappedFunction) <- new.env(parent = parent.frame())
-            environment(wrappedFunction)$nfRefClassObject <- nfRefClassObject
-            functionList[[i]] <- nfBuildCInterface(cppProj, instance = wrappedFunction, name = name)
-	}
-	return(functionList)		
-    }
-
-## nf should be a nfProcessing object OR a nimbleFunction (generator or function).
-makeCppNIMBLEfunction <- function(nf, name, debug = FALSE, debugCpp = FALSE, isNode = FALSE, where = globalenv()) { 
-    if(inherits(nf, 'nfProcessing') | inherits(nf, 'virtualNFprocessing')) {
-        if(!missing(name)) {
-            writeLines(paste('Warning, name', name, 'will be overwritten by the name of the nfProcessing object', nfp$name))
-        }
-        name <- nf$name
-        nfp <- nf
-    } else {
-        if(missing(name)) name <- deparse(substitute(nf))
-    }
-    Cname <- Rname2CppName(name)
-    if(is.nf(nf) | is.nfGenerator(nf)) {
-        virtual <- FALSE
-        if(is.nfGenerator(nf)) {
-            if(environment(nf)$virtual) virtual <- TRUE
-        }
-        if(virtual)
-            nfp <- virtualNFprocessing$new(nf, Cname)
-        else {
-            genFun <- nf_getGeneratorFunction(nf)
-            nfp <- environment(genFun)$nfProc
-            if(is.null(nfp)) {
-                nfp <- nfProcessing$new(nf, Cname)
-                environment(genFun)$nfProc <- nfp
-            } else {
-                nfp$setClassName(Cname)
-            }
-        }
-        nfp$process(debug = debug, debugCpp = debugCpp)
-    }
-
-    if(inherits(nfp, 'nfProcessing')) {
-        ans <- cppNimbleFunctionClass$new(name = Cname, nfProc = nfp, isNode = isNode, debugCpp = debugCpp) 
-        ans$buildAll(where = where)
-    } else {
-        if(inherits(nfp, 'virtualNFprocessing'))
-            ans <- cppVirtualNimbleFunctionClass(name = Cname, nfProc = nfp)
-        else
-            stop('Some problem in makeCppNIMBLEfunction')
-    }
-    
-    ans
-}
-
-nfWriteCompileAndLoadSO <- function(RFun, dirName, name = deparse(substitute(RFun)), fileName = Rname2CppName(name),
-                                    writeFiles = TRUE, compileCpp = TRUE, loadSO = TRUE, debug = FALSE, debugCpp = FALSE){
-    Cname = Rname2CppName(name)
-    NFC <- makeCppNIMBLEfunction(RFun, Cname, debug = debug, debugCpp = debugCpp)
-    cppProj <- cppProjectClass$new(dirName = dirName)
-    cppProj$addClass(NFC, Cname)
-
-    if(writeFiles){
-        cppProj$writeFiles(fileName)
-        environment(RFun)$Cwritten <- TRUE
-    }
-    if(compileCpp) {
-        cppProj$compileFile(fileName)
-        environment(RFun)$compiled <- TRUE
-    }
-    if(loadSO) {
-        cppProj$loadSO(fileName)
-        environment(RFun)$loadedSO <- TRUE
-    }
-    return(cppProj)
-}
-
-nfBuildCInterface <- function(cppProj, instance, name = deparse(substitute(RFun) )){
-	return( cppProj$cppDefs[[name]]$buildCallable(instance, cppProj$dll ) )
-}
-
