@@ -17,7 +17,7 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                             ## in predefinedNimbleLists.h and predefinedNumbleLists.cpp.
                                             return(list())
                                         } else if(name == 'EIGEN_SVDCLASS' || name == 'EIGEN_EIGENCLASS') {
-                                            ## Handle these two EITEN_ classes specially. They are implemented as an inheritance hierarchy,
+                                            ## Handle these two EIGEN_ classes specially. They are implemented as an inheritance hierarchy,
                                             ## so we cannot generate the class definitions (Nick and Perry understand why).
                                             ## Note that this branch is only ever run under the control of generateStaticCode.R to
                                             ## regenerate C++ code  in predefinedNimbleLists.h and predefinedNimbleLists.cpp.
@@ -31,6 +31,7 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                         }
                                       },
                                       buildCastPtrToNamedObjectsPtrFun = function() {
+                                          ## Creating code like:
                                           ## SEXP  nfRefClass_84_castPtrPtrToNamedObjectsPtrSEXP ( SEXP input )  {
                                           ##  return( R_MakeExternalPtr(dynamic_cast<NamedObjects*>(reinterpret_cast<nfRefClass_84*>(*static_cast<void**>(R_ExternalPtrAddr(input)))), R_NilValue, R_NilValue));
                                           ##}
@@ -45,27 +46,28 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                                                         externC = TRUE)
                                       },
                                       buildCastPtrToPtrPairFun = function() {
-                                          ## SEXP  nfRefClass_84_castDerivedPtrPtrToPairOfPtrsSEXP ( SEXP input )  {
-                                          ##   nimSmartPtrBase * ptrToSmartPtrBase;
-                                          ##   nimSmartPtr<nfRefClass_84> * ptrToSmartPtr;
-                                          ##   void * ptrToPtr;
-                                          ##   SEXP SptrToSmartPtrBase;
-                                          ##   SEXP SptrToPtr;
-                                          ##   SEXP Sans;
-                                          ##
-                                          ##   ptrToSmartPtr = static_cast<nimSmartPtr<nfRefClass_84> *>(R_ExternalPtrAddr(input));
-                                          ##   ptrToSmartPtrBase = dynamic_cast<nimSmartPtrBase*>(ptrToSmartPtr);
-                                          ##   ptrToPtr = ptrToSmartPtr->getVoidPtrToRealPtr();
-                                          ##   reinterpret_cast<nfRefClass_84*>(*static_cast<void**>(ptrToPtr))->NO_hw();
-                                          ##
-                                          ##   PROTECT(SptrToSmartPtrBase = R_MakeExternalPtr(ptrToSmartPtrBase, R_NilValue, R_NilValue));
-                                          ##   PROTECT(SptrToPtr = R_MakeExternalPtr(ptrToPtr, R_NilValue, R_NilValue));
-                                          ##   PROTECT(Sans = allocVector(VECSXP,2));
-                                          ##   SET_VECTOR_ELT(Sans,0,SptrToSmartPtrBase);
-                                          ##   SET_VECTOR_ELT(Sans,1,SptrToPtr);
-                                          ##   UNPROTECT(3);
-                                          ##   return(Sans);
-                                          ## }
+## example of code being generated:
+## SEXP  nfRefClass_84_castDerivedPtrPtrToPairOfPtrsSEXP ( SEXP input )  {
+##   nimSmartPtrBase * ptrToSmartPtrBase;
+##   nimSmartPtr<nfRefClass_84> * ptrToSmartPtr;
+##   void * ptrToPtr;
+##   SEXP SptrToSmartPtrBase;
+##   SEXP SptrToPtr;
+##   SEXP Sans;
+
+##   ptrToSmartPtr = static_cast<nimSmartPtr<nfRefClass_84> *>(R_ExternalPtrAddr(input));
+##   ptrToSmartPtrBase = dynamic_cast<nimSmartPtrBase*>(ptrToSmartPtr);
+##   ptrToPtr = ptrToSmartPtr->getVoidPtrToRealPtr();
+##   reinterpret_cast<nfRefClass_84*>(*static_cast<void**>(ptrToPtr))->NO_hw();
+ 
+##   PROTECT(SptrToSmartPtrBase = R_MakeExternalPtr(ptrToSmartPtrBase, R_NilValue, R_NilValue));
+##   PROTECT(SptrToPtr = R_MakeExternalPtr(ptrToPtr, R_NilValue, R_NilValue));
+##   PROTECT(Sans = Rf_allocVector(VECSXP,2));
+##   SET_VECTOR_ELT(Sans,0,SptrToSmartPtrBase);
+##   SET_VECTOR_ELT(Sans,1,SptrToPtr);
+##   UNPROTECT(3);
+##   return(Sans);
+## }
                                           newName <- paste0(name, "_castDerivedPtrPtrToPairOfPtrsSEXP")
 
                                           args = list(input = cppSEXP(name = 'input'))
@@ -80,7 +82,7 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                                                       'ptrToPtr = ptrToSmartPtr->getVoidPtrToRealPtr();',
                                                                       'PROTECT(SptrToSmartPtrBase = R_MakeExternalPtr(ptrToSmartPtrBase, R_NilValue, R_NilValue));',
                                                                       'PROTECT(SptrToPtr = R_MakeExternalPtr(ptrToPtr, R_NilValue, R_NilValue));'))
-                                          allocVectorLine <- cppLiteral(paste0('PROTECT(Sans = allocVector(VECSXP,', 2, '));'))
+                                          allocVectorLine <- cppLiteral(paste0('PROTECT(Sans = Rf_allocVector(VECSXP,', 2, '));'))
                                           
                                           packListLines <- cppLiteral(c('SET_VECTOR_ELT(Sans,0,SptrToSmartPtrBase);',
                                                                         'SET_VECTOR_ELT(Sans,1,SptrToPtr);'
@@ -189,8 +191,8 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                         listElementTable$addSymbol(cppSEXP(name = "S_newNimList"))
                                         listElementTable$addSymbol(cppSEXP(name = "S_listName"))
                                       
-                                        newListLine[[1]] <- substitute({PROTECT(S_listName <- allocVector(STRSXP, 1));
-                                          SET_STRING_ELT(S_listName, 0, mkChar(LISTNAME));}, 
+                                        newListLine[[1]] <- substitute({PROTECT(S_listName <- Rf_allocVector(STRSXP, 1));
+                                          SET_STRING_ELT(S_listName, 0, Rf_mkChar(LISTNAME));}, 
                                           list(LISTNAME = nimCompProc$nimbleListObj$className))
                                         newListLine[[2]] <- substitute(PROTECT(S_newNimList <- makeNewNimbleList(S_listName)),
                                                                            list())
@@ -221,8 +223,8 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                         conditionalClauseEnd <- list(quote(cppLiteral('}')))
                                         environmentCPPName <- Rname2CppName('S_.xData')  ## create SEXP for ref class environment 
                                         listElementTable$addSymbol(cppSEXP(name = environmentCPPName))
-                                        envLine <- substitute({PROTECT(ENVNAME <- allocVector(STRSXP, 1));
-                                          SET_STRING_ELT(ENVNAME, 0, mkChar(".xData"));}, 
+                                        envLine <- substitute({PROTECT(ENVNAME <- Rf_allocVector(STRSXP, 1));
+                                          SET_STRING_ELT(ENVNAME, 0, Rf_mkChar(".xData"));}, 
                                           list(ENVNAME = as.name(environmentCPPName)))
                                         
                                         for(i in seq_along(elementNames)){
@@ -232,7 +234,7 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                           conditionalLineList <- c(conditionalLineList, generateConditionalLines(nimCompProc$symTab$getSymbolObject(elementNames[i]),
                                                                                                                  listElementTable$getSymbolObject(Snames[i])))
                                           
-                                          copyToListLines[[i]] <- substitute(defineVar(install(ELEMENTNAME), VALUE, GET_SLOT(ROBJ, XDATA)),
+                                          copyToListLines[[i]] <- substitute(Rf_defineVar(Rf_install(ELEMENTNAME), VALUE, PROTECT(GET_SLOT(ROBJ, XDATA))),
                                                                              list(ELEMENTNAME = elementNames[i], VALUE = as.name(Snames[i]),
                                                                                   ROBJ = as.name('RObjectPointer'),
                                                                                   XDATA = as.name(environmentCPPName)))
@@ -242,7 +244,7 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                                                        list()))
                                         returnLine <- list(substitute(return(ROBJ),
                                                            list(ROBJ = as.name('RObjectPointer'))))
-                                        unprotectLine <- list(substitute(UNPROTECT(N), list(N = numElements + 1)))
+                                        unprotectLine <- list(substitute(UNPROTECT(N), list(N = 2 * numElements + 1)))
                                         allCode <- embedListInRbracket(c(conditionalClauseStart, list(envLine),  conditionalLineList,
                                                                          copyToListLines, setFlagLine, unprotectLine,
                                                                          conditionalClauseEnd, returnLine))
@@ -269,14 +271,14 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
 
                                         environmentCPPName <- Rname2CppName('S_.xData')  ## create SEXP for ref class environment 
                                         listElementTable$addSymbol(cppSEXP(name = environmentCPPName))
-                                        envLine <- substitute({PROTECT(ENVNAME <- allocVector(STRSXP, 1));
-                                          SET_STRING_ELT(ENVNAME, 0, mkChar(".xData"));}, 
+                                        envLine <- substitute({PROTECT(ENVNAME <- Rf_allocVector(STRSXP, 1));
+                                          SET_STRING_ELT(ENVNAME, 0, Rf_mkChar(".xData"));}, 
                                           list(ENVNAME = as.name(environmentCPPName)))
 
                                         for(i in seq_along(argNames)) {
                                           Snames[i] <- Rname2CppName(paste0('S_', argNames[i]))
                                           listElementTable$addSymbol(cppSEXP(name = Snames[i]))
-                                            copyFromListLines[[i]] <- substitute(PROTECT(SVAR <- findVarInFrame(GET_SLOT(S_nimList_, XDATA), install(ARGNAME))),
+                                            copyFromListLines[[i]] <- substitute(PROTECT(SVAR <- Rf_findVarInFrame(PROTECT(GET_SLOT(S_nimList_, XDATA)), Rf_install(ARGNAME))),
                                                                                list(ARGNAME = argNames[i], 
                                                                                     SVAR = as.name(Snames[i]),
                                                                                     XDATA = as.name(environmentCPPName)))
@@ -285,7 +287,7 @@ cppNimbleListClass <- setRefClass('cppNimbleListClass',
                                             copyLines <- c(copyLines, copyLine)
                                         }
                                         numArgs <- length(argNames)
-                                        unprotectLine <- substitute(UNPROTECT(N), list(N = numArgs + 1))
+                                        unprotectLine <- substitute(UNPROTECT(N), list(N = 2 * numArgs + 1))
                                         allCode <- embedListInRbracket(c(storeSexpLine, envLine, 
                                                                          copyFromListLines, copyLines,
                                                                          list(unprotectLine)))
