@@ -780,7 +780,6 @@ copyFromRobjectViaActiveBindings <- function(Robj, cppNames, cppCopyTypes, .self
             next
         }
         else if(cppCopyTypes[[v]] == 'nodeFxnVec') {
-            ##populateNodeFxnVec(fxnPtr = .self$.basePtr, Robject = Robj, fxnVecName = v)
             populateNodeFxnVecNew(fxnPtr = .self$.basePtr, Robject = Robj, fxnVecName = v, dll = dll) 
             next
         }
@@ -843,15 +842,15 @@ copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll)
             warning("Problem in copyFromRobject.  There is an object to be copied that is NULL.  Going to browser.", call. = FALSE)
             browser()
         }
-        if(cppCopyTypes[[v]] == 'modelVar') {
+        switch(cppCopyTypes[[v]],
+        'modelVar' = {
             modelVar <- Robj[[v]] ## this is a singleVarAccessClass created by replaceModelSingles
             Cmodel <- modelVar$model$CobjectInterface
             varName <- modelVar$var
             ##message('copying modelVar (copyFromRobject)')
             getSetModelVarPtr(v, eval(call('.Call', nimbleUserNamespace$sessionSpecificDll$getModelObjectPtr, Cmodel$.basePtr, varName)), basePtr, dll = dll)
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'nimbleFunction') {
+        },
+        'nimbleFunction' = {
             modelVar <- Robj[[v]]
             Cnf <- nf_getRefClassObject(modelVar)$.CobjectInterface ##environment(modelVar)$.CobjectInterface
             if(is.list(Cnf)) {
@@ -859,20 +858,16 @@ copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll)
             } else {
                 valueBasePtr <- Cnf$.basePtr
             }
-            ##message('copying nimbleFunction (copyFromRobject)')
             getSetNimbleFunction(v, valueBasePtr, basePtr, dll = dll)
-            ## .self[[v]] <- Cnf
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'nimbleList') {
+        },
+        'nimbleList' = {
             modelVar <- Robj[[v]]
             Cnl <- modelVar$.CobjectInterface
             vptr <- nimbleInternalFunctions$newObjElementPtr(basePtr, v, dll = dll)
             cppDef <- symTab$getSymbolObject(v)$nlProc$cppDef
             getSetNimbleList(vptr, v, Cnl, cppDef, dll = dll)
-          next
-        }
-        else if(cppCopyTypes[[v]] == 'nimPtrList') {
+        },
+        'nimPtrList' = {
             if(is.null(Robj[[v]]$contentsList)) {
                 warning('Problem in copying a nimPtrList to C++ object. The contentsList is NULL. Going to browser', call. = FALSE)
                 browser()
@@ -883,15 +878,12 @@ copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll)
             }
             modelVar <- Robj[[v]] ## This is a nimPtrList 
             Cmv <- modelVar$CobjectInterface ## This was created above in build neededObjects
-            ##message('copying nimPtrList (copyFromRobject)')
             getSetNimPtrList(v, Cmv, basePtr, dll = dll)
-            ##.self[[v]] <- Cmv
-        }
-        else if(cppCopyTypes[[v]] == 'modelValues') { ## somewhat similar to modelVar
+        },
+        'modelValues' = { ## somewhat similar to modelVar
             rModelValues <- Robj[[v]]
             Cmv <- rModelValues$CobjectInterface
             if(!Cmv$initialized) {
-                ##cat('We are copying modelValues during copyFromRobject\n')
                 k = getsize(rModelValues)
                 resize(Cmv, k)
                 vNames = rModelValues[['varNames']]
@@ -900,89 +892,58 @@ copyFromRobject <- function(Robj, cppNames, cppCopyTypes, basePtr, symTab, dll)
                 Cmv$symTab <- rModelValues$symTab
                 Cmv$initialized <- TRUE
             }
-            ##message('copying modelValues (copyFromRobject)')
             getSetModelValues(v, Cmv, basePtr, dll = dll)
-            ##            .self[[v]] <- Cmv
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'nodeFxnVec') {
+        },
+        'nodeFxnVec' = {
             populateNodeFxnVecNew(fxnPtr = basePtr, Robject = Robj, fxnVecName = v, dll = dll)
-            ## populateNodeFxnVec(fxnPtr = .self$.basePtr, Robject = Robj, fxnVecName = v) 
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'modelVarAccess'){
+        },
+        'modelVarAccess' = {
             populateManyModelVarMapAccess(fxnPtr = basePtr, Robject = Robj, manyAccessName = v, dll = dll)
-            ## populateManyModelVarMapAccess(fxnPtr = .self$.basePtr, Robject = Robj, manyAccessName = v)
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'modelValuesAccess'){
+        },
+        'modelValuesAccess' = {
             populateManyModelValuesMapAccess(fxnPtr = basePtr, Robject = Robj, manyAccessName = v, dll = dll)
-            ## populateManyModelValuesMapAccess(fxnPtr = .self$.basePtr, Robject = Robj, manyAccessName = v)
-            next
-        }
-        else if(cppCopyTypes[[v]] == "modelValuesPtr"){
+        },
+        "modelValuesPtr" = {
             curObj <- Robj[[v]]
             mvPtr = curObj$modelValues$CobjectInterface$componentExtptrs[[curObj$var]]
             getSetModelValuesPtr(v, mvPtr, basePtr, dll = dll)
-            ## .self[[v]] <<- mvPtr
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'numericList'){
+        },
+        'numericList' = {
             stop('numericList is not working\n')
-            ## rawPtr = .Call('getModelObjectPtr', .self$.basePtr, v)
-            ## .self[[v]] <<- numericList(buildType = 'C', extPtr = rawPtr)
-            ## nRows = Robj[[v]]$nRow
-            ## resize(.self[[v]], nRows)
-            ## for(i in 1:nRows){
-            ##     copyDims = max(c(1, dimOrLength[[v]][[i]]) )
-            ##     d1 = copyDims[1]
-            ##     d2 = copyDims[2]
-            ##     d3 = copyDims[3]
-            ##     setSize(.self[[v]], row = i, d1, d2, d3)
-            ##     .self[[v]][[i]] <<- Robj[[v]][[i]]
-            ## }
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'indexedNodeInfoTable') {
+        },
+        'indexedNodeInfoTable' = {
             populateIndexedNodeInfoTable(fxnPtr = basePtr, Robject = Robj, indexedNodeInfoTableName = v, dll = dll)
-        }
-        else if(cppCopyTypes[[v]] == 'characterVector') {
+        },
+        'characterVector' = {
             getSetCharacterVector(v, Robj[[v]], basePtr, dll = dll)
-            ##.self[[v]] <<- Robj[[v]]
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'characterScalar') {
+        },
+        'characterScalar' = {
             getSetCharacterScalar(v, Robj[[v]], basePtr, dll = dll)
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'numericVector') {
-            ##message('copying numericVector (copyFromRobject)')
+        },
+        'numericVector' = {
             getSetNumericVector(v, Robj[[v]], basePtr, dll = dll)
-            ##.self[[v]] <<- Robj[[v]]
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'doubleScalar') {
+        },
+        'doubleScalar' = {
             getSetDoubleScalar(v, Robj[[v]], basePtr, dll = dll)
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'integerScalar') {
+        },
+        'integerScalar' = {
             getSetIntegerScalar(v, Robj[[v]], basePtr, dll = dll)
-            next
-        }
-        else if(cppCopyTypes[[v]] == 'logicalScalar') {
+        },
+        'logicalScalar' = {
             getSetLogicalScalar(v, Robj[[v]], basePtr, dll = dll)
-            next
-        }
-        else if(!(cppCopyTypes[[v]] %in% c('copierVector'))) {
-            warning(paste0("Note: cppCopyTypes not recognized. Type = ", cppCopyTypes[[v]], "\n"), call. = FALSE)
+        },
+        { ## default:
+            if(!(cppCopyTypes[[v]] %in% c('copierVector'))) {
+                warning(paste0("Note: cppCopyTypes not recognized. Type = ", cppCopyTypes[[v]], "\n"), call. = FALSE)
+            }
         }
+        )
     }
     ## second pass is for initializations that require everything from first pass be done
     for(v in cppNames) {
         if(is.null(cppCopyTypes[[v]])) next
         if(cppCopyTypes[[v]] == 'copierVector') {
             populateCopierVector(fxnPtr = basePtr, Robject = Robj, vecName = v, dll = dll)
-            ## populateCopierVector(fxnPtr = .self$.basePtr, Robject = Robj, vecName = v)
         }
     }
 }
@@ -1474,7 +1435,7 @@ CmultiNimbleListClass <- setRefClass('CmultiNimbleListClass',
 #'
 #' @details The member data of a nimbleFunction are the objects created in \code{setup} code that are used in \code{run} code or other member functions.
 #'
-#' Whether multi-interfaces are used for nested nimbleFunctions is controlled by the \code{buildInterfacesForCompiledNestedNimbleFunctions} option in \link{nimbleOptions}.
+#' Whether multi-interfaces are used for nested nimbleFunctions is controlled by the \code{buildInterfacesForCompiledNestedNimbleFunctions} option in \code{\link{nimbleOptions}}.
 #'
 #' To see an example of a multi-interface, see \code{samplerFunctions} in a compiled MCMC interface object.
 #'
