@@ -12,15 +12,18 @@ cppOutputCalls <- c(makeCallList(binaryMidOperators, 'cppOutputMidOperator'),
                     makeCallList(c('setSize', 'initialize', 'getPtr', 'dim', 'getOffset', 'strides', 'isMap', 'mapCopy', 'setMap'), 'cppOutputMemberFunction'),
                     makeCallList(eigOtherMemberFunctionCalls, 'cppOutputEigMemberFunctionNoTranslate'),
                     makeCallList(eigProxyCallsExternalUnary, 'cppOutputEigExternalUnaryFunction'),
-                    makeCallList(c('startNimbleTimer','endNimbleTimer'), 'cppOutputMemberFunction'),
+                    makeCallList(c('startNimbleTimer','endNimbleTimer','push_back'), 'cppOutputMemberFunction'),
                     makeCallList(c('nimSeqBy','nimSeqLen', 'nimSeqByLen'), 'cppOutputCallAsIs'),
                     makeCallList(nimbleListReturningOperators, 'cppNimbleListReturningOperator'),
                     list(
+                        'next' = 'cppOutputNext',
                         eigenCast = 'cppOutputEigenCast',
+                        memberData = 'cppOutputMemberData',
                         fill = 'cppOutputEigMemberFunctionNoTranslate',
                         MAKE_FIXED_VECTOR = 'cppOutputMakeFixedVector',
                         concatenateTemp = 'cppOutputEigBlank',
                         ':' = 'cppOutputColon',
+                        '::' = 'cppOutputMidOperator',
                         size = 'cppOutputSize',
                          'for' = 'cppOutputFor',
                          'if' = 'cppOutputIfWhile',
@@ -30,6 +33,8 @@ cppOutputCalls <- c(makeCallList(binaryMidOperators, 'cppOutputMidOperator'),
                          nimSwitch = 'cppOutputNimSwitch',
                          getParam = 'cppOutputGetParam',
                          getBound = 'cppOutputGetBound',
+                        dnorm = 'cppOutputDerivDist',
+                        dpois = 'cppOutputDerivDist',
                          '(' = 'cppOutputParen',
                          resize = 'cppOutputMemberFunctionDeref',
                          nfMethod = 'cppOutputNFmethod',
@@ -54,14 +59,17 @@ cppOutputCalls <- c(makeCallList(binaryMidOperators, 'cppOutputMidOperator'),
                          as.integer = 'cppOutputCast',
                          as.numeric = 'cppOutputCast',
                          numListAccess = 'cppOutputNumList',
-                         blank = 'cppOutputBlank',
+                        blank = 'cppOutputBlank',
+                        nimVerbatim = 'cppOutputSkip',
                          callC = 'cppOutputEigBlank', ## not really eigen, but this just jumps over a layer in the parse tree
                          eigBlank = 'cppOutputEigBlank',
-                         voidPtr = 'cppOutputVoidPtr'
-                         )
+                         voidPtr = 'cppOutputVoidPtr',
+                         cppLiteral = 'cppOutputLiteral'
+                        )
                     )
 cppOutputCalls[['pow']] <-  'cppOutputPow'
 cppMidOperators <- midOperators
+cppMidOperators[['::']] <- '::'
 cppMidOperators[['%*%']] <- ' * '
 cppMidOperators[['cppMemberDereference']] <- '->'
 cppMidOperators[['nfVar']] <- '->'
@@ -80,7 +88,7 @@ nimCppKeywordsThatFillSemicolon <- c('{','for',ifOrWhile,'nimSwitch','cppLiteral
 
 ## Main function for generating C++ output
 nimGenerateCpp <- function(code, symTab = NULL, indent = '', showBracket = TRUE, asArg = FALSE) {
-    if(is.numeric(code)) return(code)
+    if(is.numeric(code)) return(if(is.nan(code)) "(nimble_NaN())" else code)
     if(is.character(code)) return(paste0('\"', gsub("\\n","\\\\n", code), '\"'))
     if(is.null(code)) return('R_NilValue')
     if(is.logical(code) ) return(if(code) 'true' else 'false')
@@ -116,6 +124,8 @@ exprName2Cpp <- function(code, symTab, asArg = FALSE) {
     }
 }
 
+cppOutputNext <- function(code, symTab) 'continue'
+
 cppOutputEigenCast <- function(code, symTab) {
     paste0( '(',nimGenerateCpp(code$args[[1]], symTab), ').cast<', code$args[[2]], '>()')
 }
@@ -135,10 +145,21 @@ cppOutputVoidPtr <- function(code, symTab) {
 
 cppOutputBlank <- function(code, symTab) NULL
 
+cppOutputSkip <- function(code, symTab) nimGenerateCpp(code$args[[1]], symTab)
+
 cppOutputEigBlank <- function(code, symTab) {
     paste0('(', nimGenerateCpp(code$args[[1]], symTab), ')')
 }
 
+cppOutputDerivDist <- function(code, symTab){
+  ###for now, use different dist c++ fn if taking derivs
+  if(identical(code$cppADCode, TRUE))
+    paste0('nimDerivs_',code$name, '(', 
+           paste0('TYPE_(',unlist(lapply(code$args[-length(code$args)], nimGenerateCpp, symTab, asArg = TRUE) ), ')', collapse = ', '), ')')
+  else
+    paste0(code$name, '(',
+           paste0(unlist(lapply(code$args, nimGenerateCpp, symTab, asArg = TRUE) ), collapse = ', '), ')')
+}
 
 cppOutputNumList <- function(code, symTab) {
     paste0( nimGenerateCpp(code$args[[1]], symTab))
@@ -173,8 +194,6 @@ cppNewNimbleList <- function(code, symTab) {
     ## This won't work for something like A$B <- nl$new()
     ## because the first arg of the caller is A$B, not a simple name
     ## But the generator info is embedded in sizeExprs
-      ##listType <- symTab$getSymbolObject(code$caller$args[[1]]$name)$templateArgs
-  ##if(is.null(listType)) listType <-  symTab$getSymbolObject(code$caller$args[[1]]$name, inherits = TRUE)$templateArgs
     listType <- code$sizeExprs$nlProc$cppDef$name
     paste0("new ", listType)
 }
@@ -237,31 +256,25 @@ cppOutputNimSwitch <- function(code, symTab) {
 }
 
 cppOutputGetParam <- function(code, symTab) {
-    ##    return(paste0(code$args[[1]]$name,'.getNodeFunctionPtrs()[0]->getParam_',code$nDim,'D_',code$type,'(', code$args[[2]]$name, ')'))
-  ##  iNodeFunction <- if(length(code$args) < 3) 0 else paste(cppMinusOne(nimDeparse(code$args[[3]])))
     if(length(code$args) < 4) {  ## code$args[[3]] is used for the paramInfo that is only used in size processing
-        ans <- paste0('getParam_',code$nDim,'D_',code$type,'(',code$args[[2]]$name,',',code$args[[1]]$name,'.getUseInfoVec()[0])')
+        ans <- paste0('getParam_',code$nDim,'D_',code$type,'(',code$args[[2]]$name,',',code$args[[1]]$name,'.getInstructions()[0])')
     } else {
         iNodeFunction <- paste(cppMinusOne(nimDeparse(code$args[[4]])))
         ans <- paste0('getParam_',code$nDim,'D_',code$type,'(',code$args[[2]]$name,',',code$args[[1]]$name,
-                      '.getUseInfoVec()[',iNodeFunction,'],', iNodeFunction ,')')
+                      '.getInstructions()[',iNodeFunction,'],', iNodeFunction ,')')
     }
     return(ans)
-##    return(paste0('getParam_',code$nDim,'D_',code$type,'(',code$args[[2]]$name,',',code$args[[1]]$name,'.getUseInfoVec()[',iNodeFunction,'])'))
 }
 
 cppOutputGetBound <- function(code, symTab) {
-    ##    return(paste0(code$args[[1]]$name,'.getNodeFunctionPtrs()[0]->getParam_',code$nDim,'D_',code$type,'(', code$args[[2]]$name, ')'))
-  ##  iNodeFunction <- if(length(code$args) < 3) 0 else paste(cppMinusOne(nimDeparse(code$args[[3]])))
     if(length(code$args) < 4) {  ## code$args[[3]] is used for the paramInfo that is only used in size processing
-        ans <- paste0('getBound_',code$nDim,'D_',code$type,'(',code$args[[2]]$name,',',code$args[[1]]$name,'.getUseInfoVec()[0])')
+        ans <- paste0('getBound_',code$nDim,'D_',code$type,'(',code$args[[2]]$name,',',code$args[[1]]$name,'.getInstructions()[0])')
     } else {
         iNodeFunction <- paste(cppMinusOne(nimDeparse(code$args[[4]])))
         ans <- paste0('getBound_',code$nDim,'D_',code$type,'(',code$args[[2]]$name,',',code$args[[1]]$name,
-                      '.getUseInfoVec()[',iNodeFunction,'],', iNodeFunction ,')')
+                      '.getInstructions()[',iNodeFunction,'],', iNodeFunction ,')')
     }
     return(ans)
-##    return(paste0('getParam_',code$nDim,'D_',code$type,'(',code$args[[2]]$name,',',code$args[[1]]$name,'.getUseInfoVec()[',iNodeFunction,'])'))
 }
 
 cppOutputEigenMapAssign <- function(code, symTab) {
@@ -294,6 +307,10 @@ cppOutputSize <- function(code, symTab) {
     paste0('static_cast<int>(', nimGenerateCpp(code$args[[1]], symTab), '.size())')
 }
 
+cppOutputMemberData <- function(code, symTab) {
+    paste0( nimGenerateCpp(code$args[[1]], symTab), '.', code$args[[2]]$name)
+}
+
 cppOutputMemberFunction <- function(code, symTab) {
     paste0( nimGenerateCpp(code$args[[1]], symTab), '.', paste0(code$name, '(', paste0(unlist(lapply(code$args[-1], nimGenerateCpp, symTab) ), collapse = ', '), ')' ))
 }
@@ -412,7 +429,7 @@ cppOutputCall <- function(code, symTab) {
 }
 
 cppOutputPow <- function(code, symTab) {
-    useStaticCase <- if(is.numeric(code$args[[2]]) ) TRUE else identical(code$args[[2]]$nDim, 0)
+    useStaticCase <- if(identical(code$cppADCode, TRUE)) FALSE else if(is.numeric(code$args[[2]]) ) TRUE else identical(code$args[[2]]$nDim, 0)
     if(useStaticCase)
         paste0(exprName2Cpp(code, symTab), '( static_cast<double>(',nimGenerateCpp(code$args[[1]], symTab, asArg = TRUE),'),', nimGenerateCpp(code$args[[2]], symTab, asArg = TRUE),')')
     else
@@ -450,3 +467,6 @@ cppOutputLiteral <- function(code, symTab) {
 cppOutputTemplate <- function(code, symTab) {
     paste0(code$args[[1]]$name, '<', paste0(unlist(lapply(code$args[-1], nimGenerateCpp, symTab, asArg = TRUE) ), collapse = ', '), '>' )
 }
+
+cppOutputLiteral <- function(code, symTab) code$args[[1]]
+
