@@ -29,6 +29,7 @@
 #' )
 #' @export
 initializeModel <- nimbleFunction(
+    name = 'initializeModel',
     setup = function(model, silent = FALSE) {
         initFunctionList <- nimbleFunctionList(nodeInit_virtual)
         iter <- 1
@@ -60,6 +61,7 @@ initializeModel <- nimbleFunction(
 nodeInit_virtual <- nimbleFunctionVirtual()
 
 checkRHSonlyInit <- nimbleFunction(
+    name = 'checkRHSonlyInit',
     contains = nodeInit_virtual,
     setup = function(model, nodes) {},
     run = function() {
@@ -69,6 +71,7 @@ checkRHSonlyInit <- nimbleFunction(
 )
 
 stochNodeInit <- nimbleFunction(
+    name = 'stochNodeInit',
     contains = nodeInit_virtual,
     setup = function(model, node, silent) {
         thisDetermNodes <- model$getDependencies(node, determOnly=TRUE)
