@@ -138,15 +138,15 @@ SEXP C_dwish_chol(SEXP x, SEXP chol, SEXP df, SEXP scale_param, SEXP return_log)
 // Cholesky matrix should be given as a numeric vector in column-major order
 //   including all n x n elements; lower-triangular elements are ignored
 {
-  if(!isMatrix(x) || !isMatrix(chol) || !isReal(x) || !isReal(chol))
+  if(!Rf_isMatrix(x) || !Rf_isMatrix(chol) || !Rf_isReal(x) || !Rf_isReal(chol))
     RBREAK("Error (C_dwish_chol): 'x' and 'chol' must be real matrices.\n")
-  if(!isReal(df) || !isReal(scale_param) || !isLogical(return_log))
+  if(!Rf_isReal(df) || !Rf_isReal(scale_param) || !Rf_isLogical(return_log))
     RBREAK("Error (C_dwish_chol): invalid input type for one of the arguments.\n");
-  int* dims = INTEGER(getAttrib(x, R_DimSymbol));
+  int* dims = INTEGER(Rf_getAttrib(x, R_DimSymbol));
   if(dims[0] != dims[1])
     RBREAK("Error (C_dwish_chol): 'x' must be a square matrix.\n");
   int p = dims[0];
-  dims = INTEGER(getAttrib(chol, R_DimSymbol));
+  dims = INTEGER(Rf_getAttrib(chol, R_DimSymbol));
   if(dims[0] != dims[1] || dims[0] != p)
    RBREAK("Error (C_dwish_chol): 'chol' must be a square matrix with the same dimensions as 'x'.\n");
 
@@ -161,7 +161,7 @@ SEXP C_dwish_chol(SEXP x, SEXP chol, SEXP df, SEXP scale_param, SEXP return_log)
     RBREAK("Error (C_dwish_chol): inconsistent degrees of freedom and dimension.\n");
   
   SEXP ans;
-  PROTECT(ans = allocVector(REALSXP, 1));  
+  PROTECT(ans = Rf_allocVector(REALSXP, 1));  
   REAL(ans)[0] = dwish_chol(c_x, c_chol, c_df, p, scale, give_log, 0); 
   UNPROTECT(1);
   return ans;
@@ -249,11 +249,11 @@ SEXP C_rwish_chol(SEXP chol, SEXP df, SEXP scale_param)
 // Cholesky matrix should be given as a numeric vector in column-major order
 //   including all n x n elements; lower-triangular elements are ignored
 {
-  if(!isMatrix(chol) || !isReal(chol))
+  if(!Rf_isMatrix(chol) || !Rf_isReal(chol))
     RBREAK("Error (Crwish_chol): 'chol' must be a real matrix.\n")
-  if(!isReal(df) || !isReal(scale_param))
+  if(!Rf_isReal(df) || !Rf_isReal(scale_param))
     RBREAK("Error (C_rwish_chol): invalid input type for one of the arguments.\n");
-  int* dims = INTEGER(getAttrib(chol, R_DimSymbol));
+  int* dims = INTEGER(Rf_getAttrib(chol, R_DimSymbol));
   if(dims[0] != dims[1])
     RBREAK("Error (C_rwish_chol): 'chol' must be a square matrix.\n");
   int p = dims[0];
@@ -270,7 +270,7 @@ SEXP C_rwish_chol(SEXP chol, SEXP df, SEXP scale_param)
   GetRNGstate(); 
 
   SEXP ans;
-  PROTECT(ans = allocVector(REALSXP, n_chol));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_chol));  
   rwish_chol(REAL(ans), c_chol, c_df, p, scale, 0);
   
   PutRNGstate();
@@ -386,7 +386,7 @@ SEXP C_dinvwish_chol(SEXP x, SEXP chol, SEXP df, SEXP scale_param, SEXP return_l
 // Cholesky matrix should be given as a numeric vector in column-major order
 //   including all n x n elements; lower-triangular elements are ignored
 {
-  if(!isReal(x) || !isReal(chol) || !isReal(df) || !isReal(scale_param) || !isLogical(return_log))
+  if(!Rf_isReal(x) || !Rf_isReal(chol) || !Rf_isReal(df) || !Rf_isReal(scale_param) || !Rf_isLogical(return_log))
     RBREAK("Error (C_dinvwish_chol): invalid input type for one of the arguments.\n");
   int p = pow(LENGTH(chol), 0.5);
   int give_log = (int) LOGICAL(return_log)[0];
@@ -400,7 +400,7 @@ SEXP C_dinvwish_chol(SEXP x, SEXP chol, SEXP df, SEXP scale_param, SEXP return_l
     RBREAK("Error (C_dinvwish_chol): inconsistent degrees of freedom and dimension.\n");
   
   SEXP ans;
-  PROTECT(ans = allocVector(REALSXP, 1));  
+  PROTECT(ans = Rf_allocVector(REALSXP, 1));  
   REAL(ans)[0] = dinvwish_chol(c_x, c_chol, c_df, p, scale, give_log, 0); 
   UNPROTECT(1);
   return ans;
@@ -491,7 +491,7 @@ SEXP C_rinvwish_chol(SEXP chol, SEXP df, SEXP scale_param)
 // Cholesky matrix should be given as a numeric vector in column-major order
 //   including all n x n elements; lower-triangular elements are ignored
 {
-  if(!isReal(chol) || !isReal(df) || !isReal(scale_param))
+  if(!Rf_isReal(chol) || !Rf_isReal(df) || !Rf_isReal(scale_param))
     RBREAK("Error (C_rinvwish_chol): invalid input type for one of the arguments.\n");
   int n_chol = LENGTH(chol);
   int p = pow(n_chol, 0.5);
@@ -506,7 +506,7 @@ SEXP C_rinvwish_chol(SEXP chol, SEXP df, SEXP scale_param)
   GetRNGstate(); 
 
   SEXP ans;
-  PROTECT(ans = allocVector(REALSXP, n_chol));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_chol));  
   rinvwish_chol(REAL(ans), c_chol, c_df, p, scale, 0); 
   
   PutRNGstate();
@@ -574,7 +574,7 @@ void rdirch(double *ans, double* alpha, int K)
 
 SEXP C_ddirch(SEXP x, SEXP alpha, SEXP return_log) 
 {
-  if(!isReal(x) || !isReal(alpha) || !isLogical(return_log)) 
+  if(!Rf_isReal(x) || !Rf_isReal(alpha) || !Rf_isLogical(return_log)) 
     RBREAK("Error (C_ddirch): invalid input type for one of the arguments.\n");
   int K = LENGTH(alpha);
   if(LENGTH(x) != K)
@@ -589,7 +589,7 @@ SEXP C_ddirch(SEXP x, SEXP alpha, SEXP return_log)
   double* c_x = REAL(x);
   double* c_alpha = REAL(alpha);
 
-  PROTECT(ans = allocVector(REALSXP, 1));  
+  PROTECT(ans = Rf_allocVector(REALSXP, 1));  
   REAL(ans)[0] = ddirch(c_x, c_alpha, K, give_log);
 
   UNPROTECT(1);
@@ -598,14 +598,14 @@ SEXP C_ddirch(SEXP x, SEXP alpha, SEXP return_log)
  
 
 SEXP C_rdirch(SEXP alpha) {
-  if(!isReal(alpha))
+  if(!Rf_isReal(alpha))
     RBREAK("Error (C_rdirch): invalid input type for the argument.\n");
   int K = LENGTH(alpha);
 
   SEXP ans;
 
   if(K == 0) {
-    PROTECT(ans = allocVector(INTSXP, 0));
+    PROTECT(ans = Rf_allocVector(INTSXP, 0));
     UNPROTECT(1);
     return ans;
   }
@@ -614,7 +614,7 @@ SEXP C_rdirch(SEXP alpha) {
 
   GetRNGstate(); 
 
-  PROTECT(ans = allocVector(REALSXP, K));  
+  PROTECT(ans = Rf_allocVector(REALSXP, K));  
   rdirch(REAL(ans), c_alpha, K);
   PutRNGstate();
   UNPROTECT(1);
@@ -693,7 +693,7 @@ void rmulti(int *ans, double size, double* prob, int K) // Calling functions nee
 
 SEXP C_dmulti(SEXP x, SEXP size, SEXP prob, SEXP return_log) 
 {
-  if(!isReal(x) || !isReal(size) || !isReal(prob) || !isLogical(return_log)) 
+  if(!Rf_isReal(x) || !Rf_isReal(size) || !Rf_isReal(prob) || !Rf_isLogical(return_log)) 
     RBREAK("Error (C_dmulti): invalid input type for one of the arguments.\n");
   int K = LENGTH(prob);
   if(LENGTH(x) != K)
@@ -719,7 +719,7 @@ SEXP C_dmulti(SEXP x, SEXP size, SEXP prob, SEXP return_log)
   if(c_size > sum + 10*DBL_EPSILON || c_size < sum - 10*DBL_EPSILON)
     RBREAK("Error (C_dmulti): sum of values is not equal to size.\n");
 
-  PROTECT(ans = allocVector(REALSXP, 1));  
+  PROTECT(ans = Rf_allocVector(REALSXP, 1));  
   REAL(ans)[0] = dmulti(c_x, c_size, c_prob, K, give_log);
 
   UNPROTECT(1);
@@ -728,14 +728,14 @@ SEXP C_dmulti(SEXP x, SEXP size, SEXP prob, SEXP return_log)
  
 
 SEXP C_rmulti(SEXP size, SEXP prob) {
-  if(!isReal(size) || !isReal(prob))
+  if(!Rf_isReal(size) || !Rf_isReal(prob))
     RBREAK("Error (C_rmulti): invalid input type for one of the arguments.\n");
   int K = LENGTH(prob);
 
   SEXP ans;
 
   if(K == 0) {
-    PROTECT(ans = allocVector(INTSXP, 0));
+    PROTECT(ans = Rf_allocVector(INTSXP, 0));
     UNPROTECT(1);
     return ans;
   }
@@ -745,7 +745,7 @@ SEXP C_rmulti(SEXP size, SEXP prob) {
 
   GetRNGstate(); 
 
-  PROTECT(ans = allocVector(INTSXP, K));  
+  PROTECT(ans = Rf_allocVector(INTSXP, K));  
   // note that if NaN set in rmulti, the INTEGER() casts it to NA
   rmulti(INTEGER(ans), c_size, c_prob, K);
   PutRNGstate();
@@ -802,7 +802,7 @@ SEXP C_dcat(SEXP x, SEXP prob, SEXP return_log)
   // this will call NIMBLE's dcat() for computation on scalars
   // prob must be a single vector of probs adding to one, but x can be a vector
 
-  if(!isReal(x) || !isReal(prob) || !isLogical(return_log)) 
+  if(!Rf_isReal(x) || !Rf_isReal(prob) || !Rf_isLogical(return_log)) 
     RBREAK("Error (C_dcat): invalid input type for one of the arguments.\n");
   int n_x = LENGTH(x);
   int K = LENGTH(prob);
@@ -818,7 +818,7 @@ SEXP C_dcat(SEXP x, SEXP prob, SEXP return_log)
   double* c_x = REAL(x);
   double* c_prob = REAL(prob);
 
-  PROTECT(ans = allocVector(REALSXP, n_x));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_x));  
   for(i = 0; i < n_x; i++) {
     REAL(ans)[i] = dcat(c_x[i], c_prob, K, give_log);
   }
@@ -828,7 +828,7 @@ SEXP C_dcat(SEXP x, SEXP prob, SEXP return_log)
 }
   
 SEXP C_rcat(SEXP n, SEXP prob) {
-  if(!isInteger(n) || !isReal(prob))
+  if(!Rf_isInteger(n) || !Rf_isReal(prob))
     RBREAK("Error (C_rcat): invalid input type for one of the arguments.\n");
   int n_values = INTEGER(n)[0];
   int K = LENGTH(prob);
@@ -837,7 +837,7 @@ SEXP C_rcat(SEXP n, SEXP prob) {
   int i;
 
   if(n_values == 0) {
-    PROTECT(ans = allocVector(INTSXP, 0));
+    PROTECT(ans = Rf_allocVector(INTSXP, 0));
     UNPROTECT(1);
     return ans;
   }
@@ -847,7 +847,7 @@ SEXP C_rcat(SEXP n, SEXP prob) {
   double* c_prob = REAL(prob);
 
   GetRNGstate(); 
-  PROTECT(ans = allocVector(INTSXP, n_values));  
+  PROTECT(ans = Rf_allocVector(INTSXP, n_values));  
 
   for(i = 0; i < n_values; i++) 
     INTEGER(ans)[i] = (int) rcat(c_prob, K);
@@ -858,6 +858,8 @@ SEXP C_rcat(SEXP n, SEXP prob) {
 
 }
   
+
+
 double dmnorm_chol(double* x, double* mean, double* chol, int n, double prec_param, int give_log, int overwrite_inputs) {
   char uplo('U');
   char transPrec('N');
@@ -924,14 +926,14 @@ SEXP C_dmnorm_chol(SEXP x, SEXP mean, SEXP chol, SEXP prec_param, SEXP return_lo
 // Cholesky matrix should be given as a numeric vector in column-major order
 //   including all n x n elements; lower-triangular elements are ignored
 {
-  if(!isMatrix(chol) || !isReal(chol))
+  if(!Rf_isMatrix(chol) || !Rf_isReal(chol))
     RBREAK("Error (C_dmnorm_chol): 'chol' must be a real matrix.\n");
-  if(!isReal(x) || !isReal(mean))
+  if(!Rf_isReal(x) || !Rf_isReal(mean))
     RBREAK("Error (C_dmnorm_chol): 'x' and 'mean' should be real valued.\n");
-  if(!isReal(prec_param) || !isLogical(return_log))
+  if(!Rf_isReal(prec_param) || !Rf_isLogical(return_log))
     RBREAK("Error (C_dmnorm_chol): invalid input type for one of the arguments.\n");
 
-  int* dims = INTEGER(getAttrib(chol, R_DimSymbol));
+  int* dims = INTEGER(Rf_getAttrib(chol, R_DimSymbol));
   if(dims[0] != dims[1])
     RBREAK("Error (C_dmnorm_chol): 'chol' must be a square matrix.\n");
   int p = dims[0];
@@ -958,7 +960,7 @@ SEXP C_dmnorm_chol(SEXP x, SEXP mean, SEXP chol, SEXP prec_param, SEXP return_lo
   } else full_mean = c_mean;
   
   SEXP ans;
-  PROTECT(ans = allocVector(REALSXP, 1));  
+  PROTECT(ans = Rf_allocVector(REALSXP, 1));  
   REAL(ans)[0] = dmnorm_chol(c_x, full_mean, c_chol, n_x, prec, give_log, 0);
   if(n_mean < n_x)
     delete [] full_mean;
@@ -1007,14 +1009,14 @@ SEXP C_rmnorm_chol(SEXP mean, SEXP chol, SEXP prec_param)
 // Cholesky matrix should be given as a numeric vector in column-major order
 //   including all n x n elements; lower-triangular elements are ignored
 {
-  if(!isMatrix(chol) || !isReal(chol))
+  if(!Rf_isMatrix(chol) || !Rf_isReal(chol))
     RBREAK("Error (C_rmnorm_chol): 'chol' should be a real matrix.\n");
-  if(!isReal(mean))
+  if(!Rf_isReal(mean))
     RBREAK("Error (C_rmnorm_chol): 'mean' should be real-valued\n");
-  if(!isReal(prec_param))
+  if(!Rf_isReal(prec_param))
     RBREAK("Error (C_rmnorm_chol): invalid input type for one of the arguments.\n");
 
-  int* dims = INTEGER(getAttrib(chol, R_DimSymbol));
+  int* dims = INTEGER(Rf_getAttrib(chol, R_DimSymbol));
   if(dims[0] != dims[1])
     RBREAK("Error (C_dmnorm_chol): 'chol' must be a square matrix.\n");
   int n_values = dims[0];
@@ -1040,7 +1042,7 @@ SEXP C_rmnorm_chol(SEXP mean, SEXP chol, SEXP prec_param)
   GetRNGstate(); 
 
   SEXP ans;
-  PROTECT(ans = allocVector(REALSXP, n_values));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_values));  
   rmnorm_chol(REAL(ans), full_mean, c_chol, n_values, prec);
 
   PutRNGstate();
@@ -1116,14 +1118,14 @@ SEXP C_dmvt_chol(SEXP x, SEXP mu, SEXP chol, SEXP df, SEXP prec_param, SEXP retu
   // Cholesky matrix should be given as a numeric vector in column-major order
   //   including all n x n elements; lower-triangular elements are ignored
 {
-  if(!isMatrix(chol) || !isReal(chol))
+  if(!Rf_isMatrix(chol) || !Rf_isReal(chol))
     RBREAK("Error (C_dmvt_chol): 'chol' must be a real matrix.\n");
-  if(!isReal(x) || !isReal(mu))
+  if(!Rf_isReal(x) || !Rf_isReal(mu))
     RBREAK("Error (C_dmvt_chol): 'x' and 'mu' should be real valued.\n");
-  if(!isReal(df) || !isReal(prec_param) || !isLogical(return_log))
+  if(!Rf_isReal(df) || !Rf_isReal(prec_param) || !Rf_isLogical(return_log))
     RBREAK("Error (C_dmvt_chol): invalid input type for one of the arguments.\n");
 
-  int* dims = INTEGER(getAttrib(chol, R_DimSymbol));
+  int* dims = INTEGER(Rf_getAttrib(chol, R_DimSymbol));
   if(dims[0] != dims[1])
     RBREAK("Error (C_dmvt_chol): 'chol' must be a square matrix.\n");
   int p = dims[0];
@@ -1151,7 +1153,7 @@ SEXP C_dmvt_chol(SEXP x, SEXP mu, SEXP chol, SEXP df, SEXP prec_param, SEXP retu
   } else full_mu = c_mu;
   
   SEXP ans;
-  PROTECT(ans = allocVector(REALSXP, 1));  
+  PROTECT(ans = Rf_allocVector(REALSXP, 1));  
   REAL(ans)[0] = dmvt_chol(c_x, full_mu, c_chol, c_df, n_x, prec, give_log, 0); 
   if(n_mu < n_x)
     delete [] full_mu;
@@ -1203,14 +1205,14 @@ SEXP C_rmvt_chol(SEXP mu, SEXP chol, SEXP df, SEXP prec_param)
   // Cholesky matrix should be given as a numeric vector in column-major order
   //   including all n x n elements; lower-triangular elements are ignored
 {
-  if(!isMatrix(chol) || !isReal(chol))
+  if(!Rf_isMatrix(chol) || !Rf_isReal(chol))
     RBREAK("Error (C_rmvt_chol): 'chol' should be a real matrix.\n");
-  if(!isReal(mu))
+  if(!Rf_isReal(mu))
     RBREAK("Error (C_rmvt_chol): 'mu' should be real-valued\n");
-  if(!isReal(prec_param))
+  if(!Rf_isReal(prec_param))
     RBREAK("Error (C_rmvt_chol): invalid input type for one of the arguments.\n");
 
-  int* dims = INTEGER(getAttrib(chol, R_DimSymbol));
+  int* dims = INTEGER(Rf_getAttrib(chol, R_DimSymbol));
   if(dims[0] != dims[1])
     RBREAK("Error (C_rmvt_chol): 'chol' must be a square matrix.\n");
   int n_values = dims[0];
@@ -1237,7 +1239,7 @@ SEXP C_rmvt_chol(SEXP mu, SEXP chol, SEXP df, SEXP prec_param)
   GetRNGstate(); 
   
   SEXP ans;
-  PROTECT(ans = allocVector(REALSXP, n_values));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_values));  
   rmvt_chol(REAL(ans), full_mu, c_chol, c_df, n_values, prec);
   
   PutRNGstate();
@@ -1311,7 +1313,7 @@ double qt_nonstandard(double p, double df, double mu, double sigma, int lower_ta
 
 
 SEXP C_dt_nonstandard(SEXP x, SEXP df, SEXP mu, SEXP sigma, SEXP return_log) {
-  if(!isReal(x) || !isReal(df) || !isReal(mu) || !isReal(sigma) || !isLogical(return_log)) 
+  if(!Rf_isReal(x) || !Rf_isReal(df) || !Rf_isReal(mu) || !Rf_isReal(sigma) || !Rf_isLogical(return_log)) 
     RBREAK("Error (C_dt_nonstandard): invalid input type for one of the arguments.");
   int n_x = LENGTH(x);
   int n_mu = LENGTH(mu);
@@ -1324,7 +1326,7 @@ SEXP C_dt_nonstandard(SEXP x, SEXP df, SEXP mu, SEXP sigma, SEXP return_log) {
     return x;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_x));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_x));  
   double* c_x = REAL(x);
   double* c_mu = REAL(mu);
   double* c_sigma = REAL(sigma);
@@ -1355,7 +1357,7 @@ SEXP C_dt_nonstandard(SEXP x, SEXP df, SEXP mu, SEXP sigma, SEXP return_log) {
   
 SEXP C_rt_nonstandard(SEXP n, SEXP df, SEXP mu, SEXP sigma) {
   // this will call R's rt() for computation on scalars
-  if(!isInteger(n) || !isReal(df) || !isReal(mu) || !isReal(sigma))
+  if(!Rf_isInteger(n) || !Rf_isReal(df) || !Rf_isReal(mu) || !Rf_isReal(sigma))
     RBREAK("Error (C_rt_nonstandard): invalid input type for one of the arguments.");
   int n_mu = LENGTH(mu);
   int n_sigma = LENGTH(sigma);
@@ -1364,7 +1366,7 @@ SEXP C_rt_nonstandard(SEXP n, SEXP df, SEXP mu, SEXP sigma) {
   SEXP ans;
     
   if(n_values == 0) {
-    PROTECT(ans = allocVector(REALSXP, 0));
+    PROTECT(ans = Rf_allocVector(REALSXP, 0));
     UNPROTECT(1);
     return ans;
   }
@@ -1374,7 +1376,7 @@ SEXP C_rt_nonstandard(SEXP n, SEXP df, SEXP mu, SEXP sigma) {
     
   GetRNGstate(); 
     
-  PROTECT(ans = allocVector(REALSXP, n_values));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_values));  
   double* c_mu = REAL(mu);
   double* c_sigma = REAL(sigma);
   double* c_df = REAL(df);
@@ -1402,7 +1404,7 @@ SEXP C_rt_nonstandard(SEXP n, SEXP df, SEXP mu, SEXP sigma) {
 }
   
 SEXP C_pt_nonstandard(SEXP q, SEXP df, SEXP mu, SEXP sigma, SEXP lower_tail, SEXP log_p) {
-  if(!isReal(q) || !isReal(df) || !isReal(mu) || !isReal(sigma) || !isLogical(lower_tail) || !isLogical(log_p))
+  if(!Rf_isReal(q) || !Rf_isReal(df) || !Rf_isReal(mu) || !Rf_isReal(sigma) || !Rf_isLogical(lower_tail) || !Rf_isLogical(log_p))
     RBREAK("Error (C_pt_nonstandard): invalid input type for one of the arguments.");
   int n_q = LENGTH(q);
   int n_mu = LENGTH(mu);
@@ -1416,7 +1418,7 @@ SEXP C_pt_nonstandard(SEXP q, SEXP df, SEXP mu, SEXP sigma, SEXP lower_tail, SEX
     return q;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_q));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_q));  
   double* c_q = REAL(q);
   double* c_mu = REAL(mu);
   double* c_sigma = REAL(sigma);
@@ -1446,7 +1448,7 @@ SEXP C_pt_nonstandard(SEXP q, SEXP df, SEXP mu, SEXP sigma, SEXP lower_tail, SEX
 }
  
 SEXP C_qt_nonstandard(SEXP p, SEXP df, SEXP mu, SEXP sigma, SEXP lower_tail, SEXP log_p) {
-  if(!isReal(p) || !isReal(df) || !isReal(mu) || !isReal(sigma) || !isLogical(lower_tail) || !isLogical(log_p))
+  if(!Rf_isReal(p) || !Rf_isReal(df) || !Rf_isReal(mu) || !Rf_isReal(sigma) || !Rf_isLogical(lower_tail) || !Rf_isLogical(log_p))
     RBREAK("Error (C_qt_nonstandard): invalid input type for one of the arguments.");
   int n_p = LENGTH(p);
   int n_mu = LENGTH(mu);
@@ -1460,7 +1462,7 @@ SEXP C_qt_nonstandard(SEXP p, SEXP df, SEXP mu, SEXP sigma, SEXP lower_tail, SEX
     return p;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_p));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_p));  
   double* c_p = REAL(p);
   double* c_mu = REAL(mu);
   double* c_sigma = REAL(sigma);
@@ -1532,7 +1534,7 @@ double rinterval(double t, double* c, int K)
 SEXP C_dinterval(SEXP x, SEXP t, SEXP c, SEXP return_log) {
   // this will call NIMBLE's dinterval() for computation on scalars
   // c must be a single vector of cutpoints; x and t can be vectors
-  if(!isReal(x) || !isReal(t) || !isReal(c) || !isLogical(return_log)) 
+  if(!Rf_isReal(x) || !Rf_isReal(t) || !Rf_isReal(c) || !Rf_isLogical(return_log)) 
     RBREAK("Error (C_dinterval): invalid input type for one of the arguments.");
   int n_x = LENGTH(x);
   int n_t = LENGTH(t);
@@ -1544,7 +1546,7 @@ SEXP C_dinterval(SEXP x, SEXP t, SEXP c, SEXP return_log) {
     return x;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_x));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_x));  
   double* c_x = REAL(x);
   double* c_t = REAL(t);
   double* c_c = REAL(c);
@@ -1568,7 +1570,7 @@ SEXP C_dinterval(SEXP x, SEXP t, SEXP c, SEXP return_log) {
 }
   
 SEXP C_rinterval(SEXP n, SEXP t, SEXP c) {
-  if(!isInteger(n) || !isReal(t) || !isReal(c))
+  if(!Rf_isInteger(n) || !Rf_isReal(t) || !Rf_isReal(c))
     RBREAK("Error (C_rinterval): invalid input type for one of the arguments.");
   int n_t = LENGTH(t);
   int K = LENGTH(c);
@@ -1576,7 +1578,7 @@ SEXP C_rinterval(SEXP n, SEXP t, SEXP c) {
   SEXP ans;
     
   if(n_values == 0) {
-    PROTECT(ans = allocVector(INTSXP, 0));
+    PROTECT(ans = Rf_allocVector(INTSXP, 0));
     UNPROTECT(1);
     return ans;
   }
@@ -1586,7 +1588,7 @@ SEXP C_rinterval(SEXP n, SEXP t, SEXP c) {
     
   GetRNGstate(); 
     
-  PROTECT(ans = allocVector(INTSXP, n_values));  
+  PROTECT(ans = Rf_allocVector(INTSXP, n_values));  
   double* c_t = REAL(t);
   double* c_c = REAL(c);
   if(n_t == 1) {
@@ -1654,7 +1656,7 @@ double qexp_nimble(double p, double rate, int lower_tail, int log_p)
 }
 
 SEXP C_dexp_nimble(SEXP x, SEXP rate, SEXP return_log) {
-  if(!isReal(x) || !isReal(rate) || !isLogical(return_log))
+  if(!Rf_isReal(x) || !Rf_isReal(rate) || !Rf_isLogical(return_log))
     RBREAK("Error (C_dexp_nimble): invalid input type for one of the arguments.");
   int n_x = LENGTH(x);
   int n_rate = LENGTH(rate);
@@ -1665,7 +1667,7 @@ SEXP C_dexp_nimble(SEXP x, SEXP rate, SEXP return_log) {
     return x;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_x));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_x));  
   double* c_x = REAL(x);
   double* c_rate = REAL(rate);
 
@@ -1688,14 +1690,14 @@ SEXP C_dexp_nimble(SEXP x, SEXP rate, SEXP return_log) {
   
 SEXP C_rexp_nimble(SEXP n, SEXP rate) {
   // this will call rexp_nimble for computation on scalars
-  if(!isInteger(n) || !isReal(rate) )
+  if(!Rf_isInteger(n) || !Rf_isReal(rate) )
     RBREAK("Error (C_rexp_nimble): invalid input type for one of the arguments.");
   int n_rate = LENGTH(rate);
   int n_values = INTEGER(n)[0];
   SEXP ans;
     
   if(n_values == 0) {
-    PROTECT(ans = allocVector(REALSXP, 0));
+    PROTECT(ans = Rf_allocVector(REALSXP, 0));
     UNPROTECT(1);
     return ans;
   }
@@ -1705,7 +1707,7 @@ SEXP C_rexp_nimble(SEXP n, SEXP rate) {
     
   GetRNGstate(); 
     
-  PROTECT(ans = allocVector(REALSXP, n_values));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_values));  
   double* c_rate = REAL(rate);
   if(n_rate == 1) {
     // if no parameter vectors, more efficient not to deal with multiple indices
@@ -1726,7 +1728,7 @@ SEXP C_rexp_nimble(SEXP n, SEXP rate) {
 }
   
 SEXP C_pexp_nimble(SEXP q, SEXP rate, SEXP lower_tail, SEXP log_p) {
-  if(!isReal(q) || !isReal(rate) || !isLogical(lower_tail) || !isLogical(log_p)) 
+  if(!Rf_isReal(q) || !Rf_isReal(rate) || !Rf_isLogical(lower_tail) || !Rf_isLogical(log_p)) 
     RBREAK("Error (C_pexp_nimble): invalid input type for one of the arguments.");
   int n_q = LENGTH(q);
   int n_rate = LENGTH(rate);
@@ -1738,7 +1740,7 @@ SEXP C_pexp_nimble(SEXP q, SEXP rate, SEXP lower_tail, SEXP log_p) {
     return q;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_q));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_q));  
   double* c_q = REAL(q);
   double* c_rate = REAL(rate);
 
@@ -1760,7 +1762,7 @@ SEXP C_pexp_nimble(SEXP q, SEXP rate, SEXP lower_tail, SEXP log_p) {
 }
   
 SEXP C_qexp_nimble(SEXP p, SEXP rate, SEXP lower_tail, SEXP log_p) {
-  if(!isReal(p) || !isReal(rate) || !isLogical(lower_tail) || !isLogical(log_p)) 
+  if(!Rf_isReal(p) || !Rf_isReal(rate) || !Rf_isLogical(lower_tail) || !Rf_isLogical(log_p)) 
     RBREAK("Error (C_qexp_nimble): invalid input type for one of the arguments.");
   int n_p = LENGTH(p);
   int n_rate = LENGTH(rate);
@@ -1772,7 +1774,7 @@ SEXP C_qexp_nimble(SEXP p, SEXP rate, SEXP lower_tail, SEXP log_p) {
     return p;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_p));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_p));  
   double* c_p = REAL(p);
   double* c_rate = REAL(rate);
 
@@ -1816,7 +1818,7 @@ double rsqrtinvgamma(double shape, double rate)
 }
 
 SEXP C_dsqrtinvgamma(SEXP x, SEXP shape, SEXP rate, SEXP return_log) {
-  if(!isReal(x) || !isReal(shape) || !isReal(rate) || !isLogical(return_log)) 
+  if(!Rf_isReal(x) || !Rf_isReal(shape) || !Rf_isReal(rate) || !Rf_isLogical(return_log)) 
     RBREAK("Error (C_dsqrtinvgamma): invalid input type for one of the arguments.");
   int n_x = LENGTH(x);
   int n_shape = LENGTH(shape);
@@ -1828,7 +1830,7 @@ SEXP C_dsqrtinvgamma(SEXP x, SEXP shape, SEXP rate, SEXP return_log) {
     return x;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_x));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_x));  
   double* c_x = REAL(x);
   double* c_shape = REAL(shape);
   double* c_rate = REAL(rate);
@@ -1854,7 +1856,7 @@ SEXP C_dsqrtinvgamma(SEXP x, SEXP shape, SEXP rate, SEXP return_log) {
 }
   
 SEXP C_rsqrtinvgamma(SEXP n, SEXP shape, SEXP rate) {
-  if(!isInteger(n) || !isReal(shape) || !isReal(rate))
+  if(!Rf_isInteger(n) || !Rf_isReal(shape) || !Rf_isReal(rate))
     RBREAK("Error (C_rsqrtinvgamma): invalid input type for one of the arguments.");
   int n_shape = LENGTH(shape);
   int n_rate = LENGTH(rate);
@@ -1862,7 +1864,7 @@ SEXP C_rsqrtinvgamma(SEXP n, SEXP shape, SEXP rate) {
   SEXP ans;
     
   if(n_values == 0) {
-    PROTECT(ans = allocVector(REALSXP, 0));
+    PROTECT(ans = Rf_allocVector(REALSXP, 0));
     UNPROTECT(1);
     return ans;
   }
@@ -1872,7 +1874,7 @@ SEXP C_rsqrtinvgamma(SEXP n, SEXP shape, SEXP rate) {
     
   GetRNGstate(); 
     
-  PROTECT(ans = allocVector(REALSXP, n_values));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_values));  
   double* c_shape = REAL(shape);
   double* c_rate = REAL(rate);
   if(n_rate == 1 && n_shape == 1 && n_rate == 1) {
@@ -1977,7 +1979,7 @@ double qinvgamma(double p, double shape, double rate, int lower_tail, int log_p)
 
 
 SEXP C_dinvgamma(SEXP x, SEXP shape, SEXP rate, SEXP return_log) {
-  if(!isReal(x) || !isReal(shape) || !isReal(rate) || !isLogical(return_log)) 
+  if(!Rf_isReal(x) || !Rf_isReal(shape) || !Rf_isReal(rate) || !Rf_isLogical(return_log)) 
     RBREAK("Error (C_dinvgamma): invalid input type for one of the arguments.");
   int n_x = LENGTH(x);
   int n_shape = LENGTH(shape);
@@ -1989,7 +1991,7 @@ SEXP C_dinvgamma(SEXP x, SEXP shape, SEXP rate, SEXP return_log) {
     return x;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_x));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_x));  
   double* c_x = REAL(x);
   double* c_shape = REAL(shape);
   double* c_rate = REAL(rate);
@@ -2015,7 +2017,7 @@ SEXP C_dinvgamma(SEXP x, SEXP shape, SEXP rate, SEXP return_log) {
 }
   
 SEXP C_rinvgamma(SEXP n, SEXP shape, SEXP rate) {
-  if(!isInteger(n) || !isReal(shape) || !isReal(rate))
+  if(!Rf_isInteger(n) || !Rf_isReal(shape) || !Rf_isReal(rate))
     RBREAK("Error (C_rinvgamma): invalid input type for one of the arguments.");
   int n_shape = LENGTH(shape);
   int n_rate = LENGTH(rate);
@@ -2023,7 +2025,7 @@ SEXP C_rinvgamma(SEXP n, SEXP shape, SEXP rate) {
   SEXP ans;
     
   if(n_values == 0) {
-    PROTECT(ans = allocVector(REALSXP, 0));
+    PROTECT(ans = Rf_allocVector(REALSXP, 0));
     UNPROTECT(1);
     return ans;
   }
@@ -2033,7 +2035,7 @@ SEXP C_rinvgamma(SEXP n, SEXP shape, SEXP rate) {
     
   GetRNGstate(); 
     
-  PROTECT(ans = allocVector(REALSXP, n_values));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_values));  
   double* c_shape = REAL(shape);
   double* c_rate = REAL(rate);
   if(n_rate == 1 && n_shape == 1 && n_rate == 1) {
@@ -2057,7 +2059,7 @@ SEXP C_rinvgamma(SEXP n, SEXP shape, SEXP rate) {
 }
   
 SEXP C_pinvgamma(SEXP q, SEXP shape, SEXP rate, SEXP lower_tail, SEXP log_p) {
-  if(!isReal(q) || !isReal(shape) || !isReal(rate) || !isLogical(lower_tail) || !isLogical(log_p))
+  if(!Rf_isReal(q) || !Rf_isReal(shape) || !Rf_isReal(rate) || !Rf_isLogical(lower_tail) || !Rf_isLogical(log_p))
     RBREAK("Error (C_pinvgamma): invalid input type for one of the arguments.");
   int n_q = LENGTH(q);
   int n_shape = LENGTH(shape);
@@ -2070,7 +2072,7 @@ SEXP C_pinvgamma(SEXP q, SEXP shape, SEXP rate, SEXP lower_tail, SEXP log_p) {
     return q;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_q));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_q));  
   double* c_q = REAL(q);
   double* c_shape = REAL(shape);
   double* c_rate = REAL(rate);
@@ -2096,7 +2098,7 @@ SEXP C_pinvgamma(SEXP q, SEXP shape, SEXP rate, SEXP lower_tail, SEXP log_p) {
 }
  
 SEXP C_qinvgamma(SEXP p, SEXP shape, SEXP rate, SEXP lower_tail, SEXP log_p) {
-  if(!isReal(p) || !isReal(shape) || !isReal(rate) || !isLogical(lower_tail) || !isLogical(log_p))
+  if(!Rf_isReal(p) || !Rf_isReal(shape) || !Rf_isReal(rate) || !Rf_isLogical(lower_tail) || !Rf_isLogical(log_p))
     RBREAK("Error (C_qinvgamma): invalid input type for one of the arguments.");
   int n_p = LENGTH(p);
   int n_shape = LENGTH(shape);
@@ -2109,7 +2111,7 @@ SEXP C_qinvgamma(SEXP p, SEXP shape, SEXP rate, SEXP lower_tail, SEXP log_p) {
     return p;
   }
     
-  PROTECT(ans = allocVector(REALSXP, n_p));  
+  PROTECT(ans = Rf_allocVector(REALSXP, n_p));  
   double* c_p = REAL(p);
   double* c_shape = REAL(shape);
   double* c_rate = REAL(rate);
@@ -2133,3 +2135,61 @@ SEXP C_qinvgamma(SEXP p, SEXP shape, SEXP rate, SEXP lower_tail, SEXP log_p) {
   UNPROTECT(1);
   return ans;
 }
+
+SEXP C_dcar_normal(SEXP x, SEXP adj, SEXP weights, SEXP num, SEXP tau, SEXP c, SEXP zero_mean, SEXP return_log) {
+  if(!Rf_isReal(x) || !Rf_isReal(adj) || !Rf_isReal(weights) || !Rf_isReal(num) || !Rf_isReal(tau) || !Rf_isReal(c) || !Rf_isReal(zero_mean) || !Rf_isLogical(return_log))
+    RBREAK("Error (C_dcar_normal): invalid input type for one of the arguments.");
+  
+  int N = LENGTH(x);
+  int L = LENGTH(adj);
+  
+  double* c_x = REAL(x);
+  double* c_adj = REAL(adj);
+  double* c_weights = REAL(weights);
+  double* c_num = REAL(num);
+  double c_tau = REAL(tau)[0];
+  int c_c = (int) REAL(c)[0];
+  int c_zero_mean = (int) REAL(zero_mean)[0];
+  int give_log = (int) LOGICAL(return_log)[0];
+  SEXP ans;
+  
+  PROTECT(ans = Rf_allocVector(REALSXP, 1));
+  REAL(ans)[0] = dcar_normal(c_x, c_adj, c_weights, c_num, c_tau, c_c, c_zero_mean, N, L, give_log);
+  
+  UNPROTECT(1);
+  return ans;
+}
+
+
+double dcar_normal(double* x, double* adj, double* weights, double* num, double tau, int c, int zero_mean, int N, int L, int give_log) {
+  // This method implements the following density calculation:
+  // p(x1, ..., xn, tau) = (tau/2/pi)^((N-c)/2) * exp(-tau/2 * sum_{i != j) w_ij (xi-xj)^2 ),
+  // where tau is precision, c = (number of islands), and N is the length of x.
+  // This is the density on an N-c dimensional space, and improper on the remaining c dimensions.
+  if(tau < 0) {
+    return R_NaN;
+  }
+  //PRINTF("c is equal to %d\n", c);
+  double lp = 0;
+  int count = 0;
+  double xi, xj;
+  for(int i = 0; i < N; i++) {
+    xi = x[i];
+    for(int j = 0; j < num[i]; j++) {
+      xj = x[ (int) adj[count] - 1 ];
+      lp += weights[count] * pow(xi-xj,2);
+      count++;
+    }
+  }
+  if(count != L) {
+    ML_ERR_return_NAN;
+  }
+  lp *= 1/2.0;     // accounts for double-summing over all (xi,xj) pairs
+  lp *= (-1/2.0) * tau;
+  lp += (N-c)/2.0 * (log(tau) - M_LN_2PI);
+  if(give_log) {
+    return(lp);
+  }
+  return(exp(lp));
+}
+
