@@ -7,19 +7,19 @@
 
 template<>
 void SEXP_2_NimArr<1>(SEXP Sn, NimArr<1, double> &ans) {
-  NIM_ASSERT2(isNumeric(Sn) || isLogical(Sn),
+  NIM_ASSERT2(Rf_isNumeric(Sn) || Rf_isLogical(Sn),
     "SEXP_2_NimArr<1, double> called for SEXP that is not a numeric or logical: actual type %s\n",
-    type2str(TYPEOF(Sn)));
+    Rf_type2str(TYPEOF(Sn)));
   int nn = LENGTH(Sn);
   NIM_ASSERT1(ans.size() == 0, "trying to reset a NimArr that was already sized\n");
   ans.setSize(nn);
-  if(isReal(Sn)) {
+  if(Rf_isReal(Sn)) {
      std::copy(REAL(Sn), REAL(Sn) + nn, ans.getPtr());
   } else {
-    NIM_ASSERT2(isInteger(Sn) || isLogical(Sn),
+    NIM_ASSERT2(Rf_isInteger(Sn) || Rf_isLogical(Sn),
       "could not handle input of type %s to SEXP_2_NimArr<1, double>\n",
-      type2str(TYPEOF(Sn)));
-    int *iSn = isInteger(Sn) ? INTEGER(Sn) : LOGICAL(Sn);
+      Rf_type2str(TYPEOF(Sn)));
+    int *iSn = Rf_isInteger(Sn) ? INTEGER(Sn) : LOGICAL(Sn);
     for(int i = 0; i < nn; ++i) {
       ans(i) = static_cast<double>(iSn[i]);
     }
@@ -29,19 +29,19 @@ void SEXP_2_NimArr<1>(SEXP Sn, NimArr<1, double> &ans) {
 // Actually this is identical to above so could be done without specialization
 template<>
 void SEXP_2_NimArr<1>(SEXP Sn, NimArr<1, int> &ans) {
-  NIM_ASSERT2(isNumeric(Sn) || isLogical(Sn),
+  NIM_ASSERT2(Rf_isNumeric(Sn) || Rf_isLogical(Sn),
     "SEXP_2_NimArr<1, int> called for SEXP that is not a numeric or logical: actual type %s\n",
-    type2str(TYPEOF(Sn)));
+    Rf_type2str(TYPEOF(Sn)));
   int nn = LENGTH(Sn);
   NIM_ASSERT1(ans.size() == 0, "trying to reset a NimArr that was already sized\n");
   ans.setSize(nn);
-  if(isReal(Sn)) {
+  if(Rf_isReal(Sn)) {
      std::copy(REAL(Sn), REAL(Sn) + nn, ans.getPtr());
   } else {
-    NIM_ASSERT2(isInteger(Sn) || isLogical(Sn),
+    NIM_ASSERT2(Rf_isInteger(Sn) || Rf_isLogical(Sn),
       "could not handle input of type %s to SEXP_2_NimArr<1, int>\n",
-      type2str(TYPEOF(Sn)));
-    int *iSn = isInteger(Sn) ? INTEGER(Sn) : LOGICAL(Sn);
+      Rf_type2str(TYPEOF(Sn)));
+    int *iSn = Rf_isInteger(Sn) ? INTEGER(Sn) : LOGICAL(Sn);
     for(int i = 0; i < nn; ++i) {
       ans(i) = static_cast<double>(iSn[i]);
     }
@@ -54,12 +54,12 @@ SEXP  EIGEN_EIGENCLASS_R::copyToSEXP (  )  {
 	SEXP S_values;
 	SEXP S_vectors;
 
-	PROTECT(S_pxData = allocVector(STRSXP, 1));
-	SET_STRING_ELT(S_pxData, 0, mkChar(".xData"));
+	PROTECT(S_pxData = Rf_allocVector(STRSXP, 1));
+	SET_STRING_ELT(S_pxData, 0, Rf_mkChar(".xData"));
 	PROTECT(S_values = NimArr_2_SEXP<1>(values));
 	PROTECT(S_vectors = NimArr_2_SEXP<2>(vectors));
-	defineVar(install("values"), S_values, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
-	defineVar(install("vectors"), S_vectors, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
+	Rf_defineVar(Rf_install("values"), S_values, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
+	Rf_defineVar(Rf_install("vectors"), S_vectors, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
 	UNPROTECT(5);
 	
 	return(RObjectPointer);
@@ -68,8 +68,8 @@ SEXP  EIGEN_EIGENCLASS_R::copyToSEXP (  )  {
 void  EIGEN_EIGENCLASS_R::createNewSEXP (  )  {
 	SEXP S_newNimList;
 	SEXP S_listName;
-	PROTECT(S_listName = allocVector(STRSXP, 1));
-	SET_STRING_ELT(S_listName, 0, mkChar("EIGEN_EIGENCLASS"));
+	PROTECT(S_listName = Rf_allocVector(STRSXP, 1));
+	SET_STRING_ELT(S_listName, 0, Rf_mkChar("EIGEN_EIGENCLASS"));
 	PROTECT(S_newNimList = makeNewNimbleList(S_listName));
 	RObjectPointer = S_newNimList;
 	UNPROTECT(2);
@@ -80,10 +80,10 @@ void  EIGEN_EIGENCLASS_R::copyFromSEXP ( SEXP S_nimList_ ) {
 	SEXP S_values;
 	SEXP S_vectors;
 	RObjectPointer =  S_nimList_;
-	PROTECT(S_pxData = allocVector(STRSXP, 1));
-	SET_STRING_ELT(S_pxData, 0, mkChar(".xData"));
-	PROTECT(S_values = findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), install("values")));
-	PROTECT(S_vectors = findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), install("vectors")));
+	PROTECT(S_pxData = Rf_allocVector(STRSXP, 1));
+	SET_STRING_ELT(S_pxData, 0, Rf_mkChar(".xData"));
+	PROTECT(S_values = Rf_findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), Rf_install("values")));
+	PROTECT(S_vectors = Rf_findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), Rf_install("vectors")));
 	SEXP_2_NimArr<1>(S_values, values);
 	SEXP_2_NimArr<2>(S_vectors, vectors);
 	UNPROTECT(5);
@@ -97,14 +97,14 @@ void  EIGEN_EIGENCLASS_R::copyFromSEXP ( SEXP S_nimList_ ) {
  	SEXP S_u;
  	SEXP S_v;
 
- 	PROTECT(S_pxData = allocVector(STRSXP, 1));
- 	SET_STRING_ELT(S_pxData, 0, mkChar(".xData"));
+ 	PROTECT(S_pxData = Rf_allocVector(STRSXP, 1));
+ 	SET_STRING_ELT(S_pxData, 0, Rf_mkChar(".xData"));
  	PROTECT(S_d = NimArr_2_SEXP<1>(d));
  	PROTECT(S_u = NimArr_2_SEXP<2>(u));
  	PROTECT(S_v = NimArr_2_SEXP<2>(v));
- 	defineVar(install("d"), S_d, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
- 	defineVar(install("u"), S_u, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
- 	defineVar(install("v"), S_v, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
+ 	Rf_defineVar(Rf_install("d"), S_d, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
+ 	Rf_defineVar(Rf_install("u"), S_u, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
+ 	Rf_defineVar(Rf_install("v"), S_v, PROTECT(GET_SLOT(RObjectPointer, S_pxData)));
  	UNPROTECT(7);
 		
  	return(RObjectPointer);
@@ -113,8 +113,8 @@ void  EIGEN_EIGENCLASS_R::copyFromSEXP ( SEXP S_nimList_ ) {
  void  EIGEN_SVDCLASS_R::createNewSEXP (  )  {
  	SEXP S_newNimList;
  	SEXP S_listName;
- 	PROTECT(S_listName = allocVector(STRSXP, 1));
- 	SET_STRING_ELT(S_listName, 0, mkChar("EIGEN_SVDCLASS"));
+ 	PROTECT(S_listName = Rf_allocVector(STRSXP, 1));
+ 	SET_STRING_ELT(S_listName, 0, Rf_mkChar("EIGEN_SVDCLASS"));
  	PROTECT(S_newNimList = makeNewNimbleList(S_listName));
  	RObjectPointer = S_newNimList;
  	UNPROTECT(2);
@@ -126,11 +126,11 @@ void  EIGEN_EIGENCLASS_R::copyFromSEXP ( SEXP S_nimList_ ) {
  	SEXP S_v;
  	SEXP S_u;
  	RObjectPointer =  S_nimList_;
- 	PROTECT(S_pxData = allocVector(STRSXP, 1));
- 	SET_STRING_ELT(S_pxData, 0, mkChar(".xData"));
- 	PROTECT(S_d = findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), install("d")));
- 	PROTECT(S_v = findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), install("v")));
- 	PROTECT(S_u = findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), install("u")));
+ 	PROTECT(S_pxData = Rf_allocVector(STRSXP, 1));
+ 	SET_STRING_ELT(S_pxData, 0, Rf_mkChar(".xData"));
+ 	PROTECT(S_d = Rf_findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), Rf_install("d")));
+ 	PROTECT(S_v = Rf_findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), Rf_install("v")));
+ 	PROTECT(S_u = Rf_findVarInFrame(PROTECT(GET_SLOT(S_nimList_, S_pxData)), Rf_install("u")));
  	SEXP_2_NimArr<1>(S_d, d);
  	SEXP_2_NimArr<2>(S_v, v);
  	SEXP_2_NimArr<2>(S_u, u);
@@ -138,8 +138,8 @@ void  EIGEN_EIGENCLASS_R::copyFromSEXP ( SEXP S_nimList_ ) {
  }
 
 SEXP C_nimEigen(SEXP S_x, SEXP S_valuesOnly, SEXP returnList) {
-  int* dims = INTEGER(getAttrib(S_x, R_DimSymbol));
-  if(!isMatrix(S_x) || dims[0] != dims[1])
+  int* dims = INTEGER(Rf_getAttrib(S_x, R_DimSymbol));
+  if(!Rf_isMatrix(S_x) || dims[0] != dims[1])
     RBREAK("Error (C_nimEigen): 'x' must be a square matrix.\n");
   NimArr<2, double> x;
   bool valuesOnly;
@@ -153,7 +153,7 @@ SEXP C_nimEigen(SEXP S_x, SEXP S_valuesOnly, SEXP returnList) {
 }
 
 SEXP C_nimSvd(SEXP S_x, SEXP S_vectors, SEXP returnList) {
-  if(!isMatrix(S_x))
+  if(!Rf_isMatrix(S_x))
     RBREAK("Error (C_nimSvd): 'x' must be a matrix.\n");
   NimArr<2, double> x;
   int vectors = SEXP_2_int(S_vectors, 0, 0);
