@@ -5,6 +5,7 @@
 #include "R.h"
 #include<Rinternals.h>
 #include<Rmath.h>
+#include<limits>
 #include<string>
 #include<time.h>
 using std::string;
@@ -34,7 +35,7 @@ class nimbleTimerClass_ {
 #endif
 
 #define PRINTF Rprintf
-#define NIMERROR error
+#define NIMERROR Rf_error
 #define RBREAK(msg) {PRINTF(msg); return(R_NilValue);}
 /* #define NIM_ASSERT(cond, ...) { if(NIM_UNLIKELY(!(cond))) { NIMERROR("Error: " __VA_ARGS__); }} */ /* future */
 #define NIM_ASSERT1(cond, msg) { if(NIM_UNLIKELY(!(cond))) { NIMERROR("Error: ", msg); }}
@@ -42,12 +43,17 @@ class nimbleTimerClass_ {
 #define NIM_ASSERT3(cond, msg, msgArg1, msgArg2) { if(NIM_UNLIKELY(!(cond))) { NIMERROR("Error: ", msg, msgArg1, msgArg2); }}
 #define NIM_ASSERT4(cond, msg, msgArg1, msgArg2, msgArg3) { if(NIM_UNLIKELY(!(cond))) { NIMERROR("Error: ", msg, msgArg1, msgArg2, msgArg3); }}
 
+#define NIM_ASSERT_SIZE(my_array, n)                                    \
+    NIM_ASSERT3(my_array.dimSize(0) == n,                               \
+                #my_array " has wrong size: expected %d, actual %d", n, \
+                my_array.dimSize(0));
+
 // code copied from nmath.h - useful utilities 
-# define MATHLIB_ERROR(fmt,x)		error(fmt,x);
-# define MATHLIB_WARNING(fmt,x)		warning(fmt,x)
-# define MATHLIB_WARNING2(fmt,x,x2)	warning(fmt,x,x2)
-# define MATHLIB_WARNING3(fmt,x,x2,x3)	warning(fmt,x,x2,x3)
-# define MATHLIB_WARNING4(fmt,x,x2,x3,x4) warning(fmt,x,x2,x3,x4)
+# define MATHLIB_ERROR(fmt,x)		Rf_error(fmt,x);
+# define MATHLIB_WARNING(fmt,x)		Rf_warning(fmt,x)
+# define MATHLIB_WARNING2(fmt,x,x2)	Rf_warning(fmt,x,x2)
+# define MATHLIB_WARNING3(fmt,x,x2,x3)	Rf_warning(fmt,x,x2,x3)
+# define MATHLIB_WARNING4(fmt,x,x2,x3,x4) Rf_warning(fmt,x,x2,x3,x4)
 
 #define ML_POSINF	R_PosInf
 #define ML_NEGINF	R_NegInf
@@ -153,4 +159,9 @@ int nimStep(double x);
 double cube(double x);
 double inprod(double v1, double v2);
 
+inline double nimble_NaN() {
+  return std::numeric_limits<double>::has_quiet_NaN
+    ? std::numeric_limits<double>::quiet_NaN()
+    : (0./0.);
+}
 #endif
