@@ -37,5 +37,7 @@ int nimArr_rinterval(double t, NimArr<1, double> &c);
 double nimArr_dinterval(double x, double t, double c, int give_log);
 int nimArr_rinterval(double t, double c);
 
+double nimArr_dcar_normal(NimArr<1, double> &x, NimArr<1, double> &adj, NimArr<1, double> &wgts, NimArr<1, double> &num, double tau, int c, int zero_mean, int give_log);
+void nimArr_rcar_normal(NimArr<1, double> &ans, NimArr<1, double> &adj, NimArr<1, double> &wgts, NimArr<1, double> &num, double tau, int c, int zero_mean);
 
 #endif
